#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : smooth_utils
# @Time         : 2021/3/18 12:27 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : 平滑函数

from meutils.pipe import *

"""
TODO: 增加可视化
"""


def exponential_decay(t, life_cycle=24 * 7, start=1, end=0):
    """牛顿冷却定律
    拟合随时间指数衰减的过程
    https://blog.csdn.net/xiaokang06/article/details/78076925
    https://blog.csdn.net/zhufenghao/article/details/80879260

    其中α为衰减常数，通过回归可计算得出。例如：指定45分钟后物体温度为初始温度的0.5，即 0.5=1×exp(-a×45)，求得α=0.1556。

    :param t: 0~delta
    :param life_cycle: 衰减时间长度/生命周期
    :param start: 起始值
    :param end: 结束值
    :return:
    """

    α = np.log(start / (end + 1e-8)) / life_cycle
    t0 = - np.log(start) / α

    decay = np.exp(- α * (t + t0))
    return decay


def walson_ctr(num_click, num_pv, z=1.96):
    """
    威尔逊
    https://mp.weixin.qq.com/s/rLP1wsS0a71q5RA7NQcjdQ
    """
    p = num_click / num_pv
    if p > 0.9:
        return 0.0

    n = num_pv

    A = p + z ** 2 / (2 * n)
    B = np.sqrt(p * (1 - p) / n + z ** 2 / (4 * (n ** 2)))
    C = z * B

    D = 1 + z ** 2 / n

    ctr = (A - C) / D

    return ctr


if __name__ == '__main__':
    for i in range(10): # SLEEP
        print(exponential_decay(i, 10, 10, 3))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : date_utils
# @Time         : 2020/11/12 11:41 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 时间/日期模块

from meutils.common import *


# pd.datetime.now().timestamp() # pd.datetime.now(cst_tz).timestamp()# 时间戳相差8*3600
# pd.read_csv(parse_dates)


def date_difference(
        fmt='%Y-%m-%d %H:%M:%S',
        start_date=None,  # Union[datetime.datetime, str, int]
        **kwargs) -> str:
    """
    start_date: datetime.datetime.today()
    days: float = ...,
    seconds: float = ...,
    microseconds: float = ...,
    milliseconds: float = ...,
    minutes: float = ...,
    hours: float = ...,
    weeks
    """
    start_date = start_date or datetime.datetime.now()

    if isinstance(start_date, (str, int)):
        start_date = datetime.datetime.strptime(str(start_date), fmt)

    date = start_date - datetime.timedelta(**kwargs)
    return date.strftime(fmt)


def timestamp2str(timestamp, fmt='%Y-%m-%d %H:%M:%S'):
    """
    t = pd.datetime.now().timestamp()
    ts = pd.Series([t]*10, name='t')

    # 时间戳 转 时间字符串
    ts = ts.map(timestamp2str) # 会有时区问题 %Y-%m-%d %H:%M:%S
    # 时间字符串 转 时间
    ts = ts.astype('datetime64[ns]') # 慢一些 pd.to_datetime(ts, errors='coerce', infer_datetime_format=True)
    ts.astype('datetime64')
    ts.astype(np.datetime64)
    # 时间 转 时间戳
    ts.map(lambda x: x.timestamp())

    # 快捷转换
    pd.to_datetime(time.time(), unit='s') # Timestamp('2021-03-18 04:26:37.819004774')

    t = pd.to_datetime('2021-04-17 23:59:59')
    t.timestamp()*1000

    :param timestamp: s
    :param fmt:
    :return:

    """
    return time.strftime(fmt, time.localtime(timestamp))


def get_nday_list(n):
    """获取过去 N 天的日期"""
    before_n_days = []
    for i in range(1, n + 1)[::-1]:
        before_n_days.append(str(datetime.date.today() - datetime.timedelta(days=i)))
    return before_n_days


def date_range(**kwargs):
    """生成一段时间区间内的日期"""
    return pd.date_range(**kwargs)


if __name__ == '__main__':
    print(date_difference(days=1))
    print(date_difference('%Y%m%d', days=1))
    print(date_difference('%Y%m%d', start_date=20210222, days=1))
    print(date_difference('%Y%m%d', days=2))
    print(date_difference(fmt='%Y%m%d', days=1))
    print(get_nday_list(30))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : celery_types
# @Time         : 2024/11/25 17:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from enum import Enum
from meutils.pipe import *


# 1. 定义标准响应模型
class TaskStatus(str, Enum):
    #: Task state is unknown (assumed pending since you know the id).
    PENDING = 'PENDING'
    #: Task was received by a worker (only used in events).
    RECEIVED = 'RECEIVED'
    #: Task was started by a worker (:setting:`task_track_started`).
    STARTED = 'STARTED'
    #: Task succeeded
    SUCCESS = 'SUCCESS'
    #: Task failed
    FAILURE = 'FAILURE'
    #: Task was revoked.
    REVOKED = 'REVOKED'
    #: Task was rejected (only used in events).
    REJECTED = 'REJECTED'
    #: Task is waiting for retry.
    RETRY = 'RETRY'
    IGNORED = 'IGNORED'


class TaskResponse(BaseModel):
    code: int
    message: str
    task_id: str = Field(default_factory=shortuuid.random)  # celery task_id
    status: TaskStatus = "PENDING"
    data: Optional[Any] = None  # 存放具体的Task结构

    created_at: str = Field(default_factory=lambda: datetime.datetime.today().isoformat())  # "2024-11-19T03:11:22.795Z"

    # 系统水印: 一般用来存轮询token ###### 不要暴露
    system_fingerprint: Optional[str] = None

    class Config:
        # 允许额外字段，增加灵活性
        extra = 'allow'

    # kling示例
    # code: int = 0
    # message: str = ""
    # request_id: str = ""
    #
    # data: Optional[Task] = None
    #
    # # 系统水印
    # system_fingerprint: Optional[str] = None


if __name__ == '__main__':
    print(TaskResponse(code=1, message='', data={'a': 1}))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : hanyuxinjie
# @Time         : 2024/9/18 13:30
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

BASE_URL = 'https://xy.siliconflow.cn'

HTML_PARSER = re.compile(r'```html(.*?)```', re.DOTALL)


# s = """
# 这是一堆文本
# ```html
# 这是一段html
# ```
# 这是一堆文本
# """
#
# print(HTML_PARSER.findall(s))


async def create(
        prompt: str = '996',
        model: str = "meta-llama/Meta-Llama-3.1-405B-Instruct",
        title: str = "🔥汉语新解",
):
    """
    "Pro/THUDM/glm-4-9b-chat"
    "Qwen/Qwen2-Math-72B-Instruct"
    “deepseek-ai/DeepSeek-V2.5”
    "meta-llama/Meta-Llama-3.1-405B-Instruct"
    """
    payload = {
        "messages": [
            {
                "role": "user",
                "content": prompt
            }
        ],
        # "chat_id": "i8yw46k",
        "model": model
    }
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=300) as client:
        async with client.stream(method="POST", url="/api/chat", json=payload) as response:
            # logger.debug(response.status_code)
            async for chunk in response.aiter_lines():
                # for chunk in "response.aiter_lines()":
                yield chunk.replace("智说新语", title)


if __name__ == '__main__':
    pass
    # model = "Pro/THUDM/glm-4-9b-chat"
    model = "Qwen/Qwen2.5-72B-Instruct"
    model = "deepseek-ai/DeepSeek-V2.5"

    # arun(create(text="火宝", model="meta-llama/Meta-Llama-3.1-405B-Instruct"))

    # arun(create(text="火宝", model="google/gemma-2-27b-it"))

    # arun(create(prompt="火宝", model=model))

    # arun(main(create(text="火宝", model="Qwen/Qwen2-Math-72B-Instruct", stream=True)))
    s = """
    这是一堆文本
    ``html
    这是一段html
    ```
    这是一堆文本
    """

    print(HTML_PARSER.findall(s))

    arun(create(prompt="996", model=model))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : word_segmentation
# @Time         : 2023/4/23 11:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

from paddlenlp import Taskflow


# 增加停用词
# 增加词频统计
# 增加共现矩阵
# 数据清洗机制 只提取中文 简繁转换 半角全角 大小写

@lru_cache()
class WordSegmentation(object):

    def __init__(self, mode='fast', user_dict=None, batch_size=32, device='cpu', filter_func=None, show_bar=False):
        """https://github.com/PaddlePaddle/PaddleNLP/blob/develop/docs/model_zoo/taskflow.md#%E4%B8%AD%E6%96%87%E5%88%86%E8%AF%8D

        :param mode:
            base 实体粒度分词，在精度和速度上的权衡，基于百度LAC
            fast 最快：实现文本快速切分，基于jieba中文分词工具
            accurate wordtag
            精确模式————最准：实体粒度切分准确度最高，基于百度解语, 精确模式基于预训练模型，更适合实体粒度分词需求，适用于知识图谱构建、企业搜索Query分析等场景中

        :param user_dict:
            在快速模式下，词典文件每一行为一个自定义item+"\t"+词频（词频可省略，词频省略则自动计算能保证分出该词的词频），暂时不支持黑名单词典（即通过设置”年“、”末“，以达到切分”年末“的目的）。
                平原上的火焰  10

            在base模式和精确模式下，词典文件每一行由一个或多个自定义item组成。
                平原上的火焰
                上 映

            在lac词性标注下，你可以通过装载自定义词典来定制化分词和词性标注结果。词典文件每一行表示一个自定义item，可以由一个单词或者多个单词组成，单词后面可以添加自定义标签，格式为item/tag，如果不添加自定义标签，则使用模型默认标签n。
                赛里木湖/LAKE
                高/a 山/n
                海拔最高

        :param filter_func:
            lambda w: w not in stopwords
        """
        self.batch_size = max(batch_size, 2)
        self.filter_func = filter_func
        self.show_bar = show_bar

        self.task_flow = Taskflow(
            "word_segmentation",
            mode=mode,
            batch_size=batch_size,
            user_dict=user_dict,
            device=device
        )

    def __call__(self, texts):
        results = texts | xbar4iter(self._task_flow, self.batch_size, show_bar=self.show_bar)

        if self.filter_func:  # 每一条都过滤
            results = results | xmap_(lambda ws: ws | xfilter_(self.filter_func))  # 重写过滤逻辑
        return results

    def _task_flow(self, texts):
        if isinstance(texts, str):
            texts = [texts]

        _ = self.task_flow(texts)
        if len(texts) == 1:
            return [_]
        return _


if __name__ == '__main__':
    print(WordSegmentation('fast', filter_func=lambda w: w not in ('的'))('我的中国人'))
    print(WordSegmentation('fast', filter_func=lambda w: w not in ('的'))(['我的中国人']))
    print(WordSegmentation('fast', filter_func=lambda w: w not in ('的'))(['我的中国人'] * 2))
    print(WordSegmentation('fast')(['我的中国人'] * 2))
    print(WordSegmentation('fast')(['我的中国人']))
    print(WordSegmentation('fast')('我的中国人'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : config
# @Time         : 2024/11/28 14:47
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from datetime import timedelta
from kombu import Exchange, Queue

# 基础配置
broker_url = 'redis://localhost:6379/0'  # 消息代理（推荐使用Redis）
broker_url = os.getenv('REDIS_URL', broker_url)

# result_backend = f"redis://localhost:6379/1"  # 结果存储
result_backend = f"{broker_url.replace('/0', '')}/1"  # 结果存储

# logger.debug(result_backend)

# 添加以下配置增加可靠性
broker_connection_retry = True
broker_connection_max_retries = 5
broker_connection_retry_on_startup = True

# 时区设置
enable_utc = False
timezone = 'Asia/Shanghai'

# 任务序列化与反序列化配置
task_serializer = 'json'
result_serializer = 'json'
accept_content = ['json']

# 并发配置
# worker_pool = 'solo'  # 工作进程池类型，可选：'prefork'、'eventlet'、'gevent'
worker_pool_restarts = True
worker_concurrency = 8  # worker并发数，一般设置为CPU核心数
worker_prefetch_multiplier = 3  # 预取任务数
worker_max_tasks_per_child = 256  # 每个worker执行多少个任务后自动重启

# 任务队列配置
task_default_queue = 'default'  # 默认队列
task_queues = (
    Queue('default', Exchange('default'), routing_key='default'),
    Queue('high_priority', Exchange('high_priority'), routing_key='high_priority'),
    Queue('low_priority', Exchange('low_priority'), routing_key='low_priority'),
)

# 任务执行设置
task_soft_time_limit = 10 * 60  # 任务软超时时间（秒）
task_time_limit = 2 * task_soft_time_limit  # 任务硬超时时间（秒）

# 任务重试配置
task_publish_retry = True  # 发布任务失败时重试
task_publish_retry_policy = {
    'max_retries': 5,  # 最大重试次数
    'interval_start': 0,  # 初始重试等待时间
    'interval_step': 2,  # 重试间隔递增步长
    'interval_max': 16,  # 最大重试间隔
}

# # 任务路由配置
# task_routes = {
#     'project.tasks.high_priority_task': {'queue': 'high_priority'},
#     'project.tasks.low_priority_task': {'queue': 'low_priority'},
# }
#
# # 定时任务配置
# beat_schedule = {
#     'task-every-30-seconds': {
#         'task': 'project.tasks.periodic_task',
#         'schedule': timedelta(seconds=30),
#     },
# }

# 日志配置
worker_log_format = '[%(asctime)s: %(levelname)s/%(processName)s] %(message)s'
worker_task_log_format = '[%(asctime)s: %(levelname)s/%(processName)s] [%(task_name)s(%(task_id)s)] %(message)s'

# 性能优化
task_acks_late = True  # 任务执行完成后再确认
task_reject_on_worker_lost = True  # worker异常关闭时任务会被重新分配
worker_disable_rate_limits = True  # 禁用任务频率限制

# 结果后端配置
result_expires = 30 * 24 * 3600  # 结果过期时间（秒）
result_persistent = True  # 结果持久化存储

# 任务执行配置
# task_always_eager = False  # 是否立即执行任务（调试用）

# 添加任务持久化配置
celery_task_store_errors_even_if_ignored = True
celery_task_ignore_result = False  # 不忽略结果
celery_task_track_started = True  # 跟踪任务状态

# Redis 持久化配置
broker_transport_options = {
    'visibility_timeout': 43200,  # 12小时
    'fanout_prefix': True,
    'fanout_patterns': True,
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2024/12/16 17:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
guidance 控制精细度 => sample_strength 0-1 数值越大生成的效果质量越好，耗时会更久
 paths: ["/mweb/v1/generate", "/mweb/v1/super_resolution", "/mweb/v1/painting", "/commerce/v1/subscription/make_unauto_order", "/commerce/v1/benefits/credit_receive", "/commerce/v1/purchase/make_order", "/mweb/v1/get_explore", "/mweb/v1/feed_short_video", "/mweb/v1/feed", "/mweb/v1/get_homepage", "/mweb/v1/get_weekly_challenge_work_list", "/mweb/v1/mget_item_info", "/mweb/v1/get_item_info", "/mweb/v1/get_history_by_ids", "/mweb/search/v1/search"],

"""

from meutils.pipe import *
from meutils.caches import rcache

from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message_for_images

from meutils.schemas.jimeng_types import BASE_URL_GLOBAL as BASE_URL, MODELS_MAP, FEISHU_URL
from meutils.schemas.image_types import ImageRequest
from meutils.schemas.task_types import TaskResponse
from meutils.apis.jimeng.common import get_headers, check_token
from meutils.apis.jimeng.files import upload_for_image
from meutils.str_utils.regular_expression import parse_url

from meutils.config_utils.lark_utils import get_next_token_for_polling
from fastapi import status, HTTPException

from fake_useragent import UserAgent

ua = UserAgent()

VERSION = "3.2.5"


#
async def create_draft_content(request: ImageRequest, token: str):
    """
    创建草稿内容
    """
    # 参考人物
    face_recognize_data = request.controls.get("face_recognize_data", {})
    image_uri = face_recognize_data.pop("image_uri", None)

    request.model = MODELS_MAP.get(request.model, MODELS_MAP["default"])

    height = width = 1328
    if 'x' in request.size:
        width, height = map(int, request.size.split('x'))

    main_component_id = str(uuid.uuid4())
    if (urls := parse_url(request.prompt)) or image_uri:  # 图生  # todo: image base64
        request.model = "high_aes_general_v20_L:general_v2.0_L"  # 2.1不支持图片编辑 某些不支持

        if image_uri:
            pass

        else:
            url = urls[-1]
            image_uri = await upload_for_image(url, token)

            request.prompt = request.prompt.replace(url, '')

        component = {
            "type": "image_base_component",
            "id": main_component_id,
            "min_version": "3.0.2",
            "generate_type": "blend",
            "aigc_mode": "workbench",
            "abilities": {
                "type": "",
                "id": str(uuid.uuid4()),
                "blend": {
                    "type": "",
                    "id": str(uuid.uuid4()),
                    "core_param": {
                        "type": "",
                        "id": str(uuid.uuid4()),
                        "model": request.model,
                        "prompt": f"##{request.prompt}",
                        "sample_strength": 0.5,
                        "image_ratio": 1,
                        "large_image_info": {
                            "type": "",
                            "id": str(uuid.uuid4()),
                            "height": height,
                            "width": width,

                            # "resolution_type": "2k"
                        },
                    },
                    "ability_list": [
                        {
                            "type": "",
                            "id": str(uuid.uuid4()),
                            "name": "byte_edit",  # bg_paint face_gan
                            "image_uri_list": [
                                image_uri
                            ],
                            "image_list": [
                                {
                                    "type": "image",
                                    "id": str(uuid.uuid4()),
                                    "source_from": "upload",
                                    "platform_type": 1,
                                    "name": "",
                                    "image_uri": image_uri,
                                    "width": 0,
                                    "height": 0,
                                    "format": "",
                                    "uri": image_uri
                                }
                            ],

                            "strength": 0.5
                        }
                    ],
                    "history_option": {
                        "type": "",
                        "id": str(uuid.uuid4()),
                    },
                    "prompt_placeholder_info_list": [
                        {
                            "type": "",
                            "id": str(uuid.uuid4()),
                            "ability_index": 0
                        }
                    ],
                    "postedit_param": {
                        "type": "",
                        "id": str(uuid.uuid4()),
                        "generate_type": 0
                    }
                }
            }
        }

        if face_recognize_data:
            face_recognize_data['name'] = "face_gan"
            component["abilities"]["blend"]["ability_list"][0].update(face_recognize_data)

    else:  # 文生

        component = {
            "type": "image_base_component",
            "id": main_component_id,
            "min_version": "3.0.2",
            "generate_type": "generate",
            "aigc_mode": "workbench",
            # 国际版
            "metadata": {
                "type": "",
                "id": str(uuid.uuid4()),
                "created_platform": 3,
                "created_platform_version": "",
                "created_time_in_ms": f"{time.time() * 1000}//1",
                "created_did": ""
            },
            "abilities": {
                "type": "",
                "id": str(uuid.uuid4()),
                "generate": {
                    "type": "",
                    "id": str(uuid.uuid4()),
                    "core_param": {
                        "type": "",
                        "id": str(uuid.uuid4()),
                        "model": request.model,
                        "prompt": request.prompt,
                        "negative_prompt": request.negative_prompt or "",
                        "seed": request.seed or 1192648548,
                        "sample_strength": request.guidance or 0.5,  # 精细度
                        "image_ratio": 1,
                        "large_image_info": {
                            "type": "",
                            "id": str(uuid.uuid4()),
                            "height": height,
                            "width": width,

                            "resolution_type": "1k"
                        }
                    },
                    "history_option": {
                        "type": "",
                        "id": str(uuid.uuid4()),
                    }
                }
            }
        }
        component = {
            "type": "image_base_component",
            "id": main_component_id,
            "min_version": "3.0.2",
            "metadata": {
                "type": "",
                "id": "85295e81-9b68-fba4-6570-81fc96c4b76e",
                "created_platform": 3,
                "created_platform_version": "",
                "created_time_in_ms": "1749271549726",
                "created_did": ""
            },
            "generate_type": "generate",
            "aigc_mode": "workbench",
            "abilities": {
                "type": "",
                "id": "91c763e9-1574-162d-1e7a-3356200dbc00",
                "generate": {
                    "type": "",
                    "id": "3862c659-7761-841a-95a1-5d02b5326e72",
                    "core_param": {
                        "type": "",
                        "id": "fcbc04a9-26ba-c890-c753-39d2a0ff9ef7",
                        "model": "high_aes_general_v30l:general_v3.0_18b",
                        "prompt": "a dog",
                        "negative_prompt": "",
                        "seed": 1192648548,
                        "sample_strength": 0.5,
                        "image_ratio": 1,
                        "large_image_info": {
                            "type": "",
                            "id": "04242843-bfee-0584-9162-a611c5ff1b54",
                            "height": 1328,
                            "width": 1328,
                            "resolution_type": "1k"
                        }
                    },
                    "history_option": {
                        "type": "",
                        "id": "4e5867c5-06e9-c101-a1a8-bb972623d701"
                    }
                }
            }
        }

    draft_content = {
        "type": "draft",
        "id": str(uuid.uuid4()),
        "min_version": "3.0.2",
        "min_features": [],
        "is_from_tsn": True,
        "version": VERSION,
        "main_component_id": main_component_id,
        "component_list": [component]
    }
    logger.debug(draft_content)
    # logger.debug(bjson(draft_content))

    return draft_content


def key_builder(*args, **kwargs):
    logger.debug(args)
    logger.debug(kwargs)

    return args[1].prompt


@retrying()
@rcache(ttl=1 * 1 * 3600, serializer="pickle", key_builder=lambda *args, **kwargs: f"{args[1].seed} {args[1].prompt}")
async def create_task(request: ImageRequest, token: Optional[str] = None):  # todo: 图片
    token = token or await get_next_token_for_polling(FEISHU_URL, check_token)

    send_message_for_images(request, __name__)

    url = "/mweb/v1/aigc_draft/generate"

    headers = get_headers(url, token)

    if "http" in request.prompt:  # 图生
        request.model = "high_aes_general_v20_L:general_v2.0_L"

    draft_content = await create_draft_content(request, token)

    logger.debug(json.dumps(draft_content))

    payload = {
        "extend": {
            "root_model": request.model,
            "template_id": ""
        },
        "submit_id": str(uuid.uuid4()),
        "metrics_extra": "{\"templateId\":\"\",\"generateCount\":1,\"promptSource\":\"custom\",\"templateSource\":\"\",\"lastRequestId\":\"\",\"originRequestId\":\"\"}",
        "draft_content": json.dumps(draft_content),
        # "http_common_info": {
        #     "aid": 513695
        # }
    }
    # logger.debug(bjson(payload))
    # payload = {"extend": {"root_model": "high_aes_general_v30l:general_v3.0_18b", "template_id": ""},
    #            "submit_id": "6e356f64-2bc3-45c7-83b6-64371bac9b64",
    #            "metrics_extra": "{\"templateId\":\"\",\"generateCount\":1,\"promptSource\":\"custom\",\"templateSource\":\"\",\"lastRequestId\":\"\",\"originRequestId\":\"\"}",
    #            "draft_content": "{\"type\":\"draft\",\"id\":\"0a47b141-8c2a-f4ce-ae51-5670688d3fdf\",\"min_version\":\"3.0.2\",\"min_features\":[],\"is_from_tsn\":true,\"version\":\"3.2.5\",\"main_component_id\":\"d7cf2d3f-db61-9775-2970-b3be2c193a71\",\"component_list\":[{\"type\":\"image_base_component\",\"id\":\"d7cf2d3f-db61-9775-2970-b3be2c193a71\",\"min_version\":\"3.0.2\",\"metadata\":{\"type\":\"\",\"id\":\"85295e81-9b68-fba4-6570-81fc96c4b76e\",\"created_platform\":3,\"created_platform_version\":\"\",\"created_time_in_ms\":\"1749271549726\",\"created_did\":\"\"},\"generate_type\":\"generate\",\"aigc_mode\":\"workbench\",\"abilities\":{\"type\":\"\",\"id\":\"91c763e9-1574-162d-1e7a-3356200dbc00\",\"generate\":{\"type\":\"\",\"id\":\"3862c659-7761-841a-95a1-5d02b5326e72\",\"core_param\":{\"type\":\"\",\"id\":\"fcbc04a9-26ba-c890-c753-39d2a0ff9ef7\",\"model\":\"high_aes_general_v30l:general_v3.0_18b\",\"prompt\":\"a dog\",\"negative_prompt\":\"\",\"seed\":1192648548,\"sample_strength\":0.5,\"image_ratio\":1,\"large_image_info\":{\"type\":\"\",\"id\":\"04242843-bfee-0584-9162-a611c5ff1b54\",\"height\":1328,\"width\":1328,\"resolution_type\":\"1k\"}},\"history_option\":{\"type\":\"\",\"id\":\"4e5867c5-06e9-c101-a1a8-bb972623d701\"}}}}]}"}

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(url, json=payload)
        response.raise_for_status()

        data = response.json()
        logger.debug(bjson(data))

    # {
    #     "ret": "1000",
    #     "errmsg": "invalid parameter",
    #     "systime": "1744354538",
    #     "logid": "20250411145538E30D2FF8347A9A710F49",
    #     "data": {
    #         "aigc_data": null,
    #         "fail_code": "",
    #         "fail_starling_key": "",
    #         "fail_starling_message": ""
    #     }
    # }

    aigc_data = (data.get("data") or {}).get("aigc_data") or {}

    logger.debug(bjson(aigc_data))

    if task_id := aigc_data.get("history_record_id"):  # bug
        return TaskResponse(task_id=task_id, system_fingerprint=token)
    else:

        # {
        #     "ret": "1018",
        #     "errmsg": "account punish limit ai generate",
        #     "systime": "1737962056",
        #     "logid": "202501271514162E3DA8ECD70A3EE1400F",
        #     "data": null
        # }

        raise HTTPException(
            status_code=status.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS,
            detail=f"可能触发内容审核，请联系管理员：{data.get('errmsg')}"
        )


@retrying(max_retries=3, min=3)
async def get_task(task_id, token):
    url = "/mweb/v1/get_history_by_ids"
    headers = get_headers(url, token)

    payload = {
        "history_ids": [
            task_id
        ]
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(url, json=payload)
        response.raise_for_status()

        logger.debug(response.text)

        task_info = {}
        if response.text:
            data = response.json()
            logger.debug(bjson(data))
            task_info = (data.get("data") or {}).get(task_id, {})
        else:
            logger.debug("走 /mweb/v1/get_history")

            data = await get_task_plus(task_id, token)

            records_list = (data.get("data") or {}).get("records_list", [])

            for record in records_list:
                if record.get("history_record_id") == task_id:
                    task_info = record

        # {'ret': '1015', 'errmsg': 'login error', 'systime': '1734524280', 'logid': '20241218201800AC3267447B287E9E6C46', 'data': None}
        item_list = task_info.get("item_list", [])  # "status": 30,

        status_code = task_info.get("status")
        fail_msg = f"""{task_info.get("fail_msg")}"""

        logger.debug(f"status: {status_code}")

        # 敏感词存储
        # if status_code != 50:
        #     send_message_for_images(task_info, __name__)

        """
        "status": 30, # 内容审核
        "status": 50,
        """

        image_data = map(lambda x: x.get("image", {}).get("large_images", []), item_list)

        task_data = sum(image_data, []) | xmap_(lambda x: {"url": x.get("image_url")})

        response = TaskResponse(
            task_id=task_id,
            data=task_data,
            message=data.get("errmsg"),
            status="success" if item_list else 'processing',
            code=status_code,
        )

        if status_code == 30:
            response.status = "fail"
            response.message = fail_msg

        return response


@retrying(max_retries=3, min=3)
async def get_task_plus(task_id, token):
    url = "/mweb/v1/get_history"
    headers = get_headers(url, token)

    params = {
        "aid": 513695,
        "da_version": "3.2.2",
        "aigc_features": "app_lip_sync",
    }

    payload = {
        "count": 20,
        "history_type_list": [
            1,
            4,
            5,
            6,
            7,
            8
        ],
        "direction": 1,
        "mode": "workbench",
        "image_info": {
            "width": 2048,
            "height": 2048,
            "format": "webp",
            "image_scene_list": [
                {
                    "scene": "smart_crop",
                    "width": 240,
                    "height": 240,
                    "uniq_key": "smart_crop-w:240-h:240",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 320,
                    "height": 320,
                    "uniq_key": "smart_crop-w:320-h:320",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 480,
                    "height": 480,
                    "uniq_key": "smart_crop-w:480-h:480",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 480,
                    "height": 320,
                    "uniq_key": "smart_crop-w:480-h:320",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 240,
                    "height": 160,
                    "uniq_key": "smart_crop-w:240-h:160",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 160,
                    "height": 213,
                    "uniq_key": "smart_crop-w:160-h:213",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 320,
                    "height": 427,
                    "uniq_key": "smart_crop-w:320-h:427",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 1080,
                    "height": 1080,
                    "uniq_key": "1080",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 900,
                    "height": 900,
                    "uniq_key": "900",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 720,
                    "height": 720,
                    "uniq_key": "720",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 480,
                    "height": 480,
                    "uniq_key": "480",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 360,
                    "height": 360,
                    "uniq_key": "360",
                    "format": "webp"
                },
                {
                    "scene": "normal",
                    "width": 2400,
                    "height": 2400,
                    "uniq_key": "2400",
                    "format": "webp"
                }
            ]
        },
        "origin_image_info": {
            "width": 96,
            "format": "webp",
            "image_scene_list": [
                {
                    "scene": "smart_crop",
                    "width": 240,
                    "height": 240,
                    "uniq_key": "smart_crop-w:240-h:240",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 320,
                    "height": 320,
                    "uniq_key": "smart_crop-w:320-h:320",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 480,
                    "height": 480,
                    "uniq_key": "smart_crop-w:480-h:480",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 480,
                    "height": 320,
                    "uniq_key": "smart_crop-w:480-h:320",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 240,
                    "height": 160,
                    "uniq_key": "smart_crop-w:240-h:160",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 160,
                    "height": 213,
                    "uniq_key": "smart_crop-w:160-h:213",
                    "format": "webp"
                },
                {
                    "scene": "smart_crop",
                    "width": 320,
                    "height": 427,
                    "uniq_key": "smart_crop-w:320-h:427",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 1080,
                    "height": 1080,
                    "uniq_key": "1080",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 900,
                    "height": 900,
                    "uniq_key": "900",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 720,
                    "height": 720,
                    "uniq_key": "720",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 480,
                    "height": 480,
                    "uniq_key": "480",
                    "format": "webp"
                },
                {
                    "scene": "loss",
                    "width": 360,
                    "height": 360,
                    "uniq_key": "360",
                    "format": "webp"
                },
                {
                    "scene": "normal",
                    "width": 2400,
                    "height": 2400,
                    "uniq_key": "2400",
                    "format": "webp"
                }
            ]
        },
        "history_option": {
            "story_id": "",
            "multi_size_image_config": [
                {
                    "height": 100,
                    "width": 100,
                    "format": "webp"
                },
                {
                    "height": 360,
                    "width": 360,
                    "format": "webp"
                },
                {
                    "height": 720,
                    "width": 720,
                    "format": "webp"
                }
            ]
        },
        "is_pack_origin": True
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(url, json=payload, params=params)
        response.raise_for_status()

        return response.json()


# @cache: todo: cache 积分异常消耗
# @cache(ttl=3600)
async def generate(request: ImageRequest):
    # logger.debug(request)

    task_response = await create_task(request)

    for i in range(1, 15):
        await asyncio.sleep(max(15 / i, 5))
        response = await get_task(task_response.task_id, task_response.system_fingerprint)
        logger.debug(f"{task_response.task_id, task_response.system_fingerprint}")
        logger.debug(response)
        if response.status.lower().startswith("fail"):
            raise HTTPException(
                status_code=status.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS,
                detail=response.message
            )

        if data := response.data:
            return {"data": data}


if __name__ == '__main__':
    # token = "eb4d120829cfd3ee957943f63d6152ed"
    token = "176b9aba6b81256b50abf08526cf311a"
    token = "7fdb556cbd1d01ba3e4a64eb0203d49d"
    image_url = "https://oss.ffire.cc/files/kling_watermark.png"

    # request = ImageRequest(prompt="做一个圣诞节的海报", size="1024x1024")
    # request = ImageRequest(prompt="https://oss.ffire.cc/files/kling_watermark.png 让她带上墨镜", size="1024x1024")

    # task = arun(create_task(request))

    # task_id = "10040025470722"

    # task_id = "10053536381698"

    # task_id = "10079694738434"

    # task_id = "10080831230210"  # 图片编辑

    # task_id = "10082971040514"
    #
    # arun(get_task(task_id, token))

    # arun(get_task(task.task_id, task.system_fingerprint))

    # task_id = "10184295086338"
    # system_fingerprint = "eb4d120829cfd3ee957943f63d6152ed"
    #
    # t1 = ("10184295086338", "eb4d120829cfd3ee957943f63d6152ed")
    # t2 = ("10184877310722", "dcf7bbc31faed9740b0bf748cd4d2c74")
    # t3 = ("10186352959490", "eb4d120829cfd3ee957943f63d6152ed")
    #
    # arun(get_task(*t3))

    from meutils.apis.jimeng.files import face_recognize

    # face_recognize = arun(face_recognize(image_url, token))
    #
    # face_recognize_data = face_recognize.get("data", {})

    # arun(generate(ImageRequest(**data)))

    # arun(generate(ImageRequest(prompt="fuck you")))
    # prompt = "A plump Chinese beauty wearing a wedding  dress revealing her skirt and underwear is swinging on the swing,Happy smile,cleavage,Exposed thighs,Spread your legs open,Extend your leg,panties,upskirt,Barefoot,sole"
    prompt = "  a dog cat in the same room       "
    # prompt = "https://oss.ffire.cc/files/kling_watermark.png 让这个女人带上墨镜，衣服换个颜色... ! "
    request = ImageRequest(prompt=prompt, size="1328x1328")
    # request = ImageRequest(prompt=prompt, size="1024x1024")

    # request = ImageRequest(prompt=prompt, size="2048*2048")

    task = arun(create_task(request, token))
    # task = arun(create_task(request, "d2d142fc877e696484cc2fc521127b36"))
    # task = arun(create_task(request, "d2d142fc877e696484cc2fc521127b36"))

    # arun(get_task(task.task_id, task.system_fingerprint))
    # arun(get_task_plus(task.task_id, task.system_fingerprint))
    # arun(get_task('16279716197378', 'd2d142fc877e696484cc2fc521127b36'))
    # arun(get_task_plus('16279716197378', 'd2d142fc877e696484cc2fc521127b36'))

    # TaskResponse(task_id='16127190069506', code=0, message=None, status='SUBMITTED', data=None,
    #              system_fingerprint='8089661372fe8db9795cc507c3049625', model=None,
    #              created_at='2025-05-06T19:49:50.933089')

    # arun(get_task("47185525857809", token))

    # arun(generate(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2024/9/11 15:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.io.files_utils import to_url

# import imgkit
#
# # HTML代码
# html_content = """
# <html>
# <body>
# <h1>Hello, World!</h1>
# <p>This is a test.</p>
# </body>
# </html>
# """
#
# # 配置
# config = imgkit.config(wkhtmltoimage='path/to/wkhtmltoimage')
#
# # 转换HTML为图片
# imgkit.from_string(html_content, 'output.jpg', config=config)
#
# print("Image saved as output.jpg")

# from html2image import Html2Image
#
# hti = Html2Image()
#
# html_content = """
# <html>
# <head>
#     <meta name="viewport" content="width=device-width, initial-scale=1">
#     <style>
#         body { font-family: Arial, sans-serif; }
#         @media (max-width: 600px) {
#             body { font-size: 14px; }
#         }
#         @media (min-width: 601px) {
#             body { font-size: 16px; }
#         }
#     </style>
# </head>
# <body>
#     <h1>Responsive Test</h1>
#     <p>This content should adapt to different screen sizes.</p>
# </body>
# </html>
# """
#
# hti.screenshot(html_str=html_content, save_as='output_desktop.png', size=(1920, 1080))
# hti.screenshot(html_str=html_content, save_as='output_mobile.png', size=(375, 812))

# from playwright.sync_api import sync_playwright
#
# # HTML代码
# html_content = """
# <html>
# <body>
# <h1>Hello, World!</h1>
# <p>This is a test.</p>
# </body>
# </html>
# """
#
# with sync_playwright() as p:
#     browser = p.chromium.launch()
#     page = browser.new_page()
#     page.set_content(html_content)
#     page.screenshot(path="output.png")
#     browser.close()
#
# print("Image saved as output.png")
#
#

# HTML 内容，包含响应式 CSS
html_content = """
<html>
<head>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <style>
        body {
            font-family: Arial, sans-serif;
            margin: 0;
            padding: 20px;
        }
        h1 {
            font-size: 24px;
        }
        p {
            font-size: 16px;
        }
        @media (max-width: 600px) {
            h1 {
                font-size: 20px;
            }
            p {
                font-size: 14px;
            }
        }
    </style>
</head>
<body>
    <h1>Hello, World!</h1>
    <p>This is a test with responsive design.</p>
</body>
</html>
"""

from playwright.sync_api import sync_playwright
from playwright.async_api import async_playwright

html_content = Path('x.html').read_text()


async def capture_screenshot(page, width, height, filename):
    await page.set_viewport_size({"width": width, "height": height})
    await page.screenshot(path=filename)
    print(f"Screenshot saved as {filename}")


async def main():
    async with async_playwright() as p:
        browser = await p.chromium.launch()
        page = await browser.new_page()
        await page.set_content(html_content)

        with tempfile.NamedTemporaryFile(suffix='.png') as file:
            logger.debug(file.name)

            # # 捕获桌面版截图
            # await capture_screenshot(page, 1280, 800, "output_desktop.png")
            #
            # # 捕获平板版截图
            # await capture_screenshot(page, 768, 1024, "output_tablet.png")

            # 捕获移动版截图
            await capture_screenshot(page, 375, 812, file.name)
            url = await to_url(file.name)
            logger.debug(url)

        await browser.close()


if __name__ == '__main__':
    arun(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : usage_utils
# @Time         : 2025/6/24 08:53
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
1. 同步任务（流 非流）
    - 按次
    - 按量
2. 异步任务
    - 按次
    - 按量
"""

from contextlib import asynccontextmanager

from meutils.pipe import *
from meutils.llm.clients import AsyncOpenAI
from meutils.apis.utils import make_request

base_url = "https://api.chatfire.cn/flux/v1"


# base_url="http://110.42.51.201:38888/flux/v1"
# base_url = "http://0.0.0.0:8000/v1/async/flux/v1"
# base_url = "https://openai-dev.chatfire.cn/usage/async/flux/v1"

async def billing_for_async_task(
        model: str = "usage-async",
        task_id: str = "123456",
        n: float = 1,
        api_key: Optional[str] = None
):
    model = model.lower().replace('/', '-')  # 统一小写 # wan-ai-wan2.1-t2v-14b
    if n := int(np.round(n)):
        tasks = [
            make_request(
                base_url=base_url,
                api_key=api_key,
                path=f"/{model}",
                payload={
                    "id": task_id,
                    # 'polling_url': f'{base_url}/get_result?id={task_id}',
                }
            )
            for i in range(n)
        ]

        _ = await asyncio.gather(*tasks)
        if _ and isinstance(_[0], str):
            raise Exception("未知模型错误，请联系管理员")

        return _


async def get_async_task(id: str = "123456"):
    # 计费
    _ = await make_request(
        base_url=base_url,
        path=f"/get_result?id={id}",

        method="GET"
    )

    return _


async def billing_for_tokens(
        model: str = "usage-chat",

        usage: Optional[dict] = None,

        api_key: Optional[str] = None,

        n: Optional[float] = None,  # 按次走以前逻辑也行
):
    """

    image_usage = {
            "input_tokens": input_tokens,
            "input_tokens_details": {
                "text_tokens": input_tokens,
                "image_tokens": 0,
            },
            "output_tokens": output_tokens,
            "total_tokens": total_tokens
        }

    usage = {
            "prompt_tokens": input_tokens,
            "completion_tokens": output_tokens,
            "total_tokens": total_tokens
        }
    """
    usage = usage or {}
    n = n and int(np.round(n))

    client = AsyncOpenAI(api_key=api_key)
    if n:
        _ = await client.images.generate(
            model=model,
            prompt="ChatfireAPI",
            n=n
        )

    elif "input_tokens" in usage:
        _ = await client.images.generate(
            model=model,
            prompt="ChatfireAPI",
            n=n,
            extra_body={"extra_fields": usage}
        )
    else:
        # todo 设计 id chatcmpl-NEdenEpvzGiKR2FfK2GmzK => 表达某些含义
        _ = await client.chat.completions.create(
            model=model,
            messages=[{"role": "user", "content": "ChatfireAPI"}],
            extra_body={"extra_body": usage}
        )
    return _


def get_billing_n(request: Union[BaseModel, dict], duration: float = 6, resolution: Optional[str] = None):
    """继续拓展其兼容性
         MINIMAX_HAILUO_02_10S_768P
    :param request:
    :param duration:
    :return:
    """
    resolution = (resolution or "500").lower().removesuffix('p')

    if isinstance(request, BaseModel):
        request = request.model_dump()

    # 数量
    num = request.get("n") or request.get("num_images") or 1

    # 时长
    dur = np.ceil(int(request.get("duration", duration)) / duration)  # 默认 6s
    durs = {"--duration 10", "--dur 10"}
    if any(i in str(request) for i in durs):
        dur = max(dur, 2)

    # 分辨率 resolution 480p 720p 1080p
    rs = request.get("resolution", resolution).lower().removesuffix('p')  # 默认 720p
    rs = np.round((int(rs) / int(resolution)) ** 2)

    # rss = {"--resolution 1080p", "--rs 1080p"}

    return num * dur * rs or 1


def get_billing_model(
        request: Union[BaseModel, dict],

        default_model: Optional[str] = None,
        default_duration: Optional[int] = None,
        default_resolution: Optional[str] = None
):
    """继续拓展其兼容性
    什么时候走这个逻辑？
    """
    if isinstance(request, BaseModel):
        request = request.model_dump()

    model = (request.get("model") or request.get("model_name") or default_model or "undefined model").lower()

    # 某些模型不需要匹配，单模型计费
    if model.startswith(("veo", "jimeng", "cogvideo")):  # todo 后续再拓展
        return model

    duration = request.get("duration", default_duration or "5")
    resolution = request.get("resolution", default_resolution or "720p")

    # 火山
    if any(i in str(request).lower() for i in {"--duration 10", "--dur 10"}):
        duration = 10

    for rs in {"480p", "720p", "1080p"}:
        if any(i in str(request).lower() for i in {f"--resolution {rs}", f"--rs {rs}"}):
            resolution = rs

    return f"{model}_{duration}s_{resolution}"


@asynccontextmanager
async def billing_flow_for_async_task(
        model: str = "usage-async",
        task_id: str = "123456",
        n: float = 1,
        api_key: Optional[str] = None
):
    if n:  # 计费
        await billing_for_async_task(model, task_id=task_id, n=n, api_key=api_key)
        yield

    else:  # 不计费
        a = yield
        logger.debug(a)


@asynccontextmanager
async def billing_flow_for_tokens(
        model: str = "usage-chat",

        usage: Optional[dict] = None,  # None就是按次

        api_key: Optional[str] = None,
):
    await billing_for_tokens(model, usage=usage, api_key=api_key)

    yield


if __name__ == '__main__':
    # arun(create_usage_for_tokens())
    # usage = {
    #     "input_tokens": 1,
    #     "input_tokens_details": {
    #         "text_tokens": 1,
    #         "image_tokens": 0,
    #     },
    #     "output_tokens": 100,
    #     "total_tokens": 101
    # }
    # n = 1
    usage = {
        "prompt_tokens": 1000, "completion_tokens": 100,  # "total_tokens": 2000,
    }
    arun(billing_for_tokens(model="tokens", usage=usage))

    # arun(create_usage_for_async_task(task_id="task_id", n=1))

    model = "async"
    # model = "fal-ai/model1"
    task_id = f"{model}-{int(time.time())}"
    # model = "doubao-seedance-1-0-pro-250528"
    model = "Wan-AI/Wan2.1-T2V-14B"

    # arun(billing_for_async_task(model, task_id=task_id, n=3))

    # data = {
    #     "model": "doubao-seedance-1-0-pro-250528",
    #     "content": [
    #         {
    #             "type": "text",
    #             "text": "多个镜头。一名侦探进入一间光线昏暗的房间。他检查桌上的线索，手里拿起桌上的某个物品。镜头转向他正在思索。 --ratio 16:9"
    #         }
    #     ]
    # }
    #
    # print(get_billing_n(data))

    #
    # arun(get_async_task(task_id))
    #
    # arun(get_async_task(f"{task_id}-Ready", status="Ready"))

    # arun(get_async_task('chatfire-123456-Ready-1'))

    # {
    #   "id": "chatfire-1750769977.856766",
    #   "result": {},
    #   "status": "Error",
    #   "details": {
    #     "xx": [
    #       "xxxx"
    #     ]
    #   },
    #   "progress": 99
    # }

    data = {
        "model": "veo3",
        "prompt": "女人飞上天了",
        "images": [
            "https://oss.ffire.cc/files/kling_watermark.png"
        ],
        "enhance_prompt": True
    }
    # print(get_billing_model(data, default_resolution=""))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2023/7/21 10:48
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 爬虫学习playwright  https://blog.csdn.net/wx17343624830/article/details/130622056
# https://blog.51cto.com/u_15800928/7735545
# https://news.sohu.com/a/685709792_121368355


from playwright.async_api import Playwright, async_playwright


async def run(playwright: Playwright) -> None:
    browser = await playwright.chromium.launch(headless=True)
    context = await browser.new_context()
    page = await context.new_page()
    await page.goto(
        "https://mp.weixin.qq.com/s?__biz=Mzg2MjIwODc3Mw==&amp;mid=2247493349&amp;idx=1&amp;sn=e21830aa26c956be38a5b47a33cfa2e7&amp;chksm=ce09f08ff97e7999caf268e06510e4ab00bca6d0e89a6e96f2ff1207cc57cea698aa3b973d36&amp;mpshare=1&amp;scene=1&amp;srcid=1101LIjE8Yh1t8wfX7gu3JQQ&amp;sharer_shareinfo=a31a8170ed734f0c59b1713c3f20ec57&amp;sharer_shareinfo_first=a31a8170ed734f0c59b1713c3f20ec57#wechat_redirect")
    # page.wait_for_load_state('networkidle')
    # page.pdf(path='p.pdf')

    # print(page.title())
    # html = page.content()
    # page.wait_for_timeout(5000)
    # print(page.query_selector_all("""//text()"""))

    # page.screenshot(full_page=True, path='screenshot.png')
    # page.locator('//*[@id="img-content"]').screenshot(path='screenshot11.png')  # 带标题
    # page.locator("#js_content").screenshot(path='screenshot22.png')  # 不带标题

    blog = page.locator("#js_content")
    print(await page.locator('//*[@id="img-content"]').inner_text())

    # ---------------------
    await context.close()
    await browser.close()


async def main():
    async with async_playwright() as p:
        await run(p)

if __name__ == '__main__':
    import asyncio
    asyncio.run(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kling_types
# @Time         : 2024/9/24 10:15
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.notice.feishu import send_message as _send_message

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/ad8c3010-7706-4ba0-b945-645c492e535b"
)

STATUSES = {
    5: "submitted",
    10: "processing",
    99: "succeed",
    50: "failed",

    6: "The uploaded image size should be greater than 300PX",
    7: 'NSFW：输入的提示词包含敏感词',
    8: "NSFW：上传图片包含敏感信息",  # 上传图片包含敏感信息

    400: "非法请求",
    500: "内部系统繁忙",

    "submitted": "SUBMITTED",
    "processing": "IN_PROGRESS",
    "succeed": "SUCCESS",
    "failed": "FAILURE",

}

API_BASE_URL = "https://api.klingai.com"
API_FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=LGVKwN"


class TryOnRequest(BaseModel):
    model_name: str = "kolors-virtual-try-on-v1"
    """
    ● 支持传入图片Base64编码或图片URL（确保可访问）
    ● 图片格式支持.jpg / .jpeg / .png
    ● 图片文件大小不能超过10MB，图片分辨率不小于300*300px
    """
    human_image: str = "https://oss.ffire.cc/files/try-on.png"
    cloth_image: str = "https://oss.ffire.cc/files/x.png"

    callback_url: Optional[str] = None

    class Config:
        frozen = True


class Config(BaseModel):
    """包含六个字段，用于指定摄像机的运动或变化"""
    """水平运镜，可选，取值范围：[-10, 10]"""
    horizontal: Optional[int] = 0
    """水平摇镜，可选，取值范围：[-10, 10]"""
    pan: Optional[int] = 0
    """旋转运镜，可选，取值范围：[-10, 10]"""
    roll: Optional[int] = 0
    """垂直摇镜，可选，取值范围：[-10, 10]"""
    tilt: Optional[int] = 0
    """垂直运镜，可选，取值范围：[-10, 10]"""
    vertical: Optional[int] = 0
    """变焦，可选，取值范围：[-10, 10]"""
    zoom: Optional[int] = 0


class CameraControl(BaseModel):
    """控制摄像机运动的协议，可选，未指定则智能匹配"""
    """包含六个字段，用于指定摄像机的运动或变化"""
    config: Config = Config()
    type: str = "simple"  # empty


class VideoRequest(BaseModel):
    model: Optional[str] = "kling-v1"

    """生成视频的模式，可选，枚举值：std（高性能）或 pro（高表现）"""
    mode: Optional[str] = "std"

    """正向文本提示，必须，不能超过500个字符"""
    prompt: str

    """生成视频的画面纵横比，可选，枚举值：16:9, 9:16, 1:1"""
    aspect_ratio: Optional[str] = "1:1"

    """控制摄像机运动的协议，可选，未指定则智能匹配"""
    camera_control: CameraControl = CameraControl()

    """生成视频的自由度，可选，值越大，相关性越强，取值范围：[0,1]"""
    cfg_scale: Optional[float] = None

    """生成视频时长，单位秒，可选，枚举值：5，10"""
    duration: Optional[int] = 5

    """负向文本提示，可选，不能超过200个字符"""
    negative_prompt: Optional[str] = None

    # 图生视频多出来的参数
    """参考图像，必须，支持Base64编码或图片URL，支持.jpg / .jpeg / .png格式，大小不能超过10MB，分辨率不小于300*300px"""
    image: Optional[str] = None

    """参考图像 - 尾帧控制，可选，支持Base64编码或图片URL，支持.jpg / .jpeg / .png格式，大小不能超过10MB，分辨率不小于300*300px"""
    image_tail: Optional[str] = None

    """生成视频数量，可选，取值范围：[1,4]"""
    n: int = 1


class ImageRequest(BaseModel):
    model: Optional[str] = None

    """正向文本提示，必须，不能超过500个字符"""
    prompt: str
    """生成图片的纵横比，可选，枚举值：16:9, 9:16, 1:1, 4:3, 3:4, 3:2, 2:3"""
    aspect_ratio: Optional[str] = None

    """生成图片数量，可选，取值范围：[1,9]"""
    n: int = 1

    """参考图片，可选，支持Base64编码或图片URL，支持.jpg / .jpeg / .png格式，大小不能超过10MB，图片分辨率不小于300*300px。
    Base64仅提供编码部分，data:image/png;base64,后面的部分。
    """
    image: Optional[str] = None

    """生成过程中对用户上传图片的参考强度，可选，取值范围：[0,1]"""
    image_fidelity: Optional[float] = None

    """负向文本提示，可选，不能超过200个字符"""
    negative_prompt: Optional[str] = None

    """本次任务结果回调通知地址，可选"""
    callback_url: Optional[str] = None


# {
#     "code": 0,
#     "message": "string",
#     "request_id": "string",
#     "data": {
#         "task_id": "string",
#         "task_status": "string",
#         "created_at": 0,
#         "updated_at": 0
#     }
# }

class TaskResult(BaseModel):
    """视频ID，系统生成"""
    id: str

    """视频URL"""
    url: str

    """视频时长，单位ms"""
    duration: int


class Task(BaseModel):
    """任务ID，系统生成"""
    task_id: str

    """任务状态，枚举值：submitted（已提交）、processing（处理中）、succeed（成功）、failed（失败）"""
    task_status: str = "processing"

    """任务状态信息，具体定义状态信息"""
    task_status_msg: Optional[str] = None

    """任务创建时间，Unix时间戳、单位ms"""
    created_at: int = Field(default_factory=lambda: int(time.time() * 1000))

    """任务更新时间，Unix时间戳、单位ms"""
    updated_at: int = Field(default_factory=lambda: int(time.time() * 1000))

    """任务最终结果"""
    task_result: Optional[dict] = None


class TaskResponse(BaseModel):
    code: int = 0
    message: str = ""
    request_id: str = ""

    data: Optional[Task] = None

    # 系统水印
    system_fingerprint: Optional[str] = None


# {
#     "code": 0,
#     "message": "string",
#     "request_id": "string",
#     "data": {
#         "task_id": "string",
#         "task_status": "string",
#         "created_at": 0,
#         "updated_at": 0
#     }
# }


# {
#     "code": 0,
#     "message": "string",
#     "request_id": "string",
#     "data": {
#         "task_id": "string",
#         "task_status": "string",
#         "task_status_msg": "string",
#         "created_at": 1722769557708,
#         "updated_at": 1722769557708,
#         "task_result": {
#             "videos": [
#                 {
#                     "id": "string",
#                     "url": "string",
#                     "duration": "string"
#                 }
#             ]
#         }
#     }
# }


if __name__ == '__main__':
    print(TaskResponse(data=Task(task_id='xx')).model_dump(exclude_none=True))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : haimian
# @Time         : 2024/8/2 15:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json

from meutils.pipe import *
from meutils.schemas.haimian_types import HaimianRequest, HaimianCustomRequest
from meutils.schemas.task_types import Task
from meutils.apis.proxy.ips import get_one_proxy

from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=ax1BQH"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)

# url = "https://www.haimian.com/jd/api/v1/generate/lyric2song?app_name=goat&aid=588628&app_id=588628&app_version=1.0.0.397&channel=online&region=CN&device_platform=web&msToken=hXy3dUqC1FoEKSDhn7h9el2p5Iegrf86G-71wZAL10-HEZxjZWHZdb2iZCsMomcIymhvKkFNFlj6SW0q6M6x-mw3P2UPKMHp73nUYMouCfpdo-_0paLZlgj9XCjCAA%3D%3D&a_bogus=QvW0BfwvDiVpDDmR5RoLfY3quOWwYdR50ajLMDgPEpBKOg39HMOl9exEoNs4RkbjN4%2FkIejjy4hbO3xprQQJ8Hwf7Wsx%2F2CZs640t-Pg-nSSs1feeLbQrsJx-kz5Feep5JV3EcvhqJKczbEk09Cn5iIlO6ZCcHgjxiSmtn3Fv-S%3D"

url = "https://www.haimian.com/jd/api/v1/generate/lyric2song"
BASE_URL = "https://www.haimian.com/jd/api/v1"


# www.haimian.com => "all.ffire.cc/haimian"

# {
#     "status_code": 10000008,
#     "status_info": {
#         "status_msg": "已超过每天音乐生成上限，请明天再试",
#         "ts": 0,
#         "log_id": ""
#     },
#     "data": null
# }

@retrying()
async def create_task(request: HaimianRequest, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL)

    headers = {
        'Cookie': token,
        # 'msToken': '7i-GsUjnIpzgFr47RfcSu-SHno90VQJNNq-G7UZuTGZkh6kG8WKiIPKRN6n3pp8YCVQhYQNbWq3mDk_FqEd8oCtiypv8tTQo6QBinmFRbRp3IcHPvX4mybl6zXK0QQ==',
        # 'fg_uid': 'RID20241125145034C2DBB2C389869EC7EFBD',

        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/130.0.0.0 Safari/537.36"
    }
    params = {
        # 'app_name': 'goat',
        # 'aid': '588628',
        # 'app_id': '588628',
        # 'app_version': '1.0.0.731',
        # 'channel': 'online',
        # 'region': 'CN',
        # 'device_platform': 'web',
        # 'msToken': '7i-GsUjnIpzgFr47RfcSu-SHno90VQJNNq-G7UZuTGZkh6kG8WKiIPKRN6n3pp8YCVQhYQNbWq3mDk_FqEd8oCtiypv8tTQo6QBinmFRbRp3IcHPvX4mybl6zXK0QQ==',
        # 'a_bogus': 'mXWZQ5gvDiVkhDEm5A5LfY3quu1gY0tr0ajLMDgPB%2FxY-y39HMP09exYtXz4TyEjNG%2FpIejjy4hbT3KkrQQrMZwf9WhE%2F2ApsDSDeM32soDys1feejusnUhimkU-taBB-k1UrO7hqvKcKbup09Cj4vIAP6ZeaHgjxiSmtn3FvlL%3D'
    }
    payload = request.model_dump()
    # proxies = await get_one_proxy()
    proxies = {
        # 'http://': 'http://154.12.35.201:38443', 'https://': 'http://154.12.35.201:38443'
    }
    logger.debug(proxies)
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30,
                                 proxies=proxies) as client:
        response = await client.post(f"/generate/lyric2song/", params=params, json=payload)

        response.raise_for_status()
        logger.info(response.text)
        logger.info(response.status_code)

        data = response.json()['data']
        task_ids = jsonpath.jsonpath(data, "$..tasks..task_id")
        return Task(id=f"haimian-{task_ids | xjoin(',')}", data=data, system_fingerprint=token)


async def get_task(task_id, token):
    task_id = task_id.split("-")[-1]
    params = [('task_ids', task_id) for task_id in task_id.split(',')]

    headers = {
        'Cookie': token,
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.get("/generate/tasks/info", params=params)
        if response.is_success:
            return response.json()


@alru_cache(ttl=30)
@retrying()
async def generate_lyrics(prompt: str = "写一首夏日晚风的思念的歌", token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL)

    payload = {
        "input": prompt,
        "type": 0
    }
    headers = {
        'Cookie': token,
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.post("/generate/lyric_tool_stream", json=payload)
        if response.is_success:
            return json_repair.repair_json(response.text, return_objects=True)
        # response: httpx.Response
        # async with client.stream("POST", "/generate/lyric_tool_stream", json=payload) as response:
        #     async for i in response.aiter_lines():
        #         print(i)


if __name__ == '__main__':
    token = None
    request = HaimianRequest(
        prompt="古风歌曲，男生唱的，乐器：古筝、箫、二胡、木鱼、鼓。\n演唱风格：空灵婉转、情感投入。\n歌词高级的古风感，整体构造成仙气飘飘的感觉乐器：琵琶、笛、扬琴、编钟。\n演唱风格：柔情细腻、悠远绵长。",
        batch_size=3
    )
    arun(create_task(request, token))

    # task_id = "haimian-WlAPDQggva"
    # token = arun(get_next_token_for_polling(FEISHU_URL))
    # arun(get_task(task_id, token))

    # arun(generate_lyrics())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : wechat
# @Time         : 2021/6/7 11:17 上午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : https://wechatpy.readthedocs.io/zh_CN/master/work/client.html

from meutils.pipe import *
from urllib.parse import urljoin
from wechatpy.enterprise import WeChatClient as _WeChatClient


class WeChatClient(_WeChatClient):
    API_BASE_URL = 'https://qyapi.weixin.qq.com/cgi-bin/'
    if is_open('08:35.72.81.291'[::-1]):
        API_BASE_URL = f"http://{'08:35.72.81.291'[::-1]}/cgi-bin/"

    def __init__(
            self,
            corp_id,
            secret,
            api_base_url=None,
            agent_id=None,
            access_token=None,
            session=None, timeout=None, auto_retry=True, **kwargs
    ):
        self.corp_id = corp_id
        self.secret = secret
        self.agent_id = agent_id
        self.API_BASE_URL = api_base_url or 'https://qyapi.weixin.qq.com/cgi-bin/'

        super().__init__(
            corp_id, secret, access_token, session, timeout, auto_retry
        )
        # NESC
        if is_open('08:35.72.81.291'[::-1]):
            self.API_BASE_URL = f"http://{'08:35.72.81.291'[::-1]}/cgi-bin/"

    # @ttl_cache(ttl=60 * 60)
    def fetch_access_token(self):
        return self._fetch_access_token(
            url=urljoin(self.API_BASE_URL, '/cgi-bin/gettoken'),
            params={
                'corpid': self.corp_id,
                'corpsecret': self.secret
            }
        )

    @staticmethod
    def name2id(name='AI小分队'):
        from meutils.hash_utils import murmurhash
        return murmurhash(name)


@background_task
def nesc_wechat(
        title='',
        content='',
        chat_id=325257180,
        corp_id='ww3c6024bb94ecef59',
        secret='empKNMx-RSgd4tK6uzVA56qCl1QY6eErRdSb7Hr5vyQ',
        agent_id='1000041'
):
    wc = WeChatClient(corp_id=corp_id, secret=secret, agent_id=agent_id)
    # wc.appchat.send_msg()
    wc.appchat.send(
        chat_id,
        'textcard',
        **{'title': title, 'description': content, 'url': 'https://github.com/yuanjie-ai/ChatLLM'}
    )


if __name__ == '__main__':
    # # 公网测试
    # corp_id = 'wwc18433f3075302e4'
    # secret = 'iL_8JXBoB5vFITCcOk2-EvP6TcOnVCjZI1LRw8vidtEE'
    # agent_id = '1000002'
    # api_base_url = None
    #
    # # 内外AI：
    corp_id = ''
    secret = ''
    agent_id = '1000041'
    api_base_url = 'https://qywxlocal.xx.cn:7443/cgi-bin/'

    wc = WeChatClient(corp_id, secret, api_base_url)
    name = '常态化巡查通知'
    chat_id = wc.name2id(name)
    # wc.appchat.create(chat_id=chat_id, name=name, owner='YuanJie', user_list=['YuanJie', 'yayoYan'])
    # _ = wc.appchat.create(chat_id=chat_id, name=name, owner=7683, user_list=[7683, 9147])
    # # 7560 离职：wechatpy.exceptions.WeChatClientException: Error code: 60111, message: userid not found [logid:]
    #
    # # wc.appchat.create(chat_id=chat_id, name=name, owner=7683, user_list=[7683, 7689])
    # wc.appchat.send_text(chat_id, f"{name}#chat_id: {chat_id}\n{_}")
    # wc.appchat.send_text(chat_id, f"{name}#chat_id: {chat_id}\n{_}")
    wc.appchat.send_text(chat_id, '# 我是个机器人🤖')

    # wc.appchat.update(chat_id, add_user_list=[7838])
    # # wc.appchat.send(chat_id, 'textcard', **{'title': 'Title', 'description': 'description', 'url': 'http://'})
    # # wc.appchat.send_text(chat_id, '# 我是个机器人🤖')
    # # wc.appchat.send(chat_id, 'markdown', content='# 我是个机器人🤖') # 不支持

    # corp_id, secret = os.getenv("D_CORP_SECRET").split('|')
    # wc = _WeChatClient(corp_id, secret)
    # name = '常态化巡查通知'
    # chat_id = wc.name2id(name)
    # # chat_id = 10000
    # _ = wc.appchat.create(chat_id=chat_id, name=name,
    #                       owner='Betterme',
    #                       # user_list=[7683, 7689]
    #                       )
    # wc.appchat.send_text(chat_id, f"{name}#chat_id: {chat_id}\n{_}")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : video
# @Time         : 2024/7/26 12:03
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling

from meutils.schemas.video_types import VideoRequest
from meutils.schemas.image_types import ImageRequest, ImagesResponse
from meutils.str_utils.regular_expression import parse_url

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)

from zhipuai import ZhipuAI
from zhipuai.types.video import VideoObject, VideoResult

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=5i64gO"


@retrying(max_retries=3, predicate=lambda r: r.task_status is None)
async def create_task(request: VideoRequest, api_key: Optional[str] = None):
    api_key = api_key or await get_next_token_for_polling(FEISHU_URL, from_redis=False)

    client = ZhipuAI(api_key=api_key)  # 请填写您自己的APIKey
    response = client.videos.generations(
        model=request.model,
        prompt=request.prompt,
        extra_body={
            "image_url": request.image_url
        }
    )

    response = VideoObject.construct(**response.model_dump(), system_fingerprint=api_key)
    response.id = f"cogvideox-{response.id}"
    # response
    logger.debug(response)
    logger.debug(response.task_status)

    # response
    return response


async def get_task(task_id, api_key):
    task_id = isinstance(task_id, str) and task_id.split("-", maxsplit=1)[-1]

    client = ZhipuAI(api_key=api_key)  # 请填写您自己的APIKey

    response = client.videos.retrieve_videos_result(
        id=task_id
    )
    return response


async def generate(request: ImageRequest, n: int = 30):  # 兼容dalle3
    request.model = "cogvideox-flash"
    # request.duration = 10 # 模型后缀

    url = None
    if urls := parse_url(request.prompt):
        url = urls[-1]
        request.prompt = request.prompt.replace(url, "")

    request = VideoRequest(image_url=url, **request.model_dump(exclude_none=True))
    task_response = await create_task(request)

    for i in range(n):
        await asyncio.sleep(max(1., n / (i + 1)))
        response = await get_task(task_response.id, task_response.system_fingerprint)

        logger.debug(response)
        if response.task_status == "SUCCESS" or response.video_result:
            return ImagesResponse(data=response.model_dump()["video_result"])


# {
#     "model": "cogvideox",
#     "request_id": "8868902201637896192",
#     "task_status": "SUCCESS",
#     "video_result": [
#         {
#             "cover_image_url": "https://sfile.chatglm.cn/testpath/video_cover/4d3c5aad-8c94-5549-93b7-97af6bd353c6_cover_0.png",
#             "url": "https://sfile.chatglm.cn/testpath/video/4d3c5aad-8c94-5549-93b7-97af6bd353c6_0.mp4"
#         }
#     ]
# }
# }

# VideoResult
# VideoObject(id='340817255120805129017958079360725371', model='cogvideox', video_result=None, task_status='PROCESSING', request_id='9017958079360725355', system_fingerprint='cf36b5587554cc3d34abd59ce252b19b.zzJ4co4kNEWPSbnm')

# VideoResult
if __name__ == '__main__':
    api_key = "88b82799f3234a5aad130b0f74c7eb85.tBMTRh0h1IqbvMaw"
    # api_key = "7d10426c06afa81e8d7401d97781249c.DbqlSsicRtaUdKXI"  # 新号
    # api_key = "e21bd630f681c4d90b390cd609720483.WSFVgA3Kk1wNCX0mN"

    request = VideoRequest(
        model='cogvideox-flash',
        # model='cogvideox'

    )
    r = arun(create_task(request, api_key=api_key))
    pass

    request = ImageRequest(prompt="https://oss.ffire.cc/files/kling_watermark.png 让这个女人笑起来")

    arun(generate(request, n=30))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : vidu_video
# @Time         : 2024/7/31 08:59
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo 测试 https://oss-shanghai.sanwubeixin.cn/ios/110580373/20241126133701.png
# https://www.capitalonline.net/zh-cn/support/openapi/index.html

import asyncio

from meutils.pipe import *
from meutils.decorators.retry import retrying

from meutils.schemas.vidu_types import BASE_URL, UPLOAD_BASE_URL, ViduRequest, ViduUpscaleRequest
from meutils.schemas.task_types import TaskType, Task, FileTask

from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling, get_spreadsheet_values

# FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=l3VpZf"
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=V84y6z"
FEISHU_URL_VIP = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=EYgZ8c"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


async def upload(file, token: Optional[str] = None, vip: bool = False):  # todo: 统一到 file object
    token = token or await get_next_token_for_polling(FEISHU_URL_VIP)

    token = token.strip(";Shunt=").strip("; Shunt=")
    logger.debug(token)

    payload = {"scene": "vidu"}
    headers = {
        "Cookie": token  # ;Shunt= 居然失效
    }
    async with httpx.AsyncClient(base_url=UPLOAD_BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post("/files/uploads", json=payload)
        if response.is_success:
            logger.debug(response.json())
            file_id = response.json()['id']
            put_url = response.json()['put_url']  # 图片url

            response = await client.put(put_url, content=file, headers=headers)
            etag = response.headers.get('Etag')

            payload = {"id": file_id, "etag": etag}
            response = await client.put(f"/files/uploads/{file_id}/finish", json=payload)
            logger.debug(response.text)
            logger.debug(response.status_code)
            logger.debug(put_url.split('?')[0])
            if response.is_success:
                uri = response.json()['uri']
                return FileTask(id=file_id, url=put_url, system_fingerprint=token)

            response.raise_for_status()


@retrying(max_retries=8, max=8, predicate=lambda r: r is True, title=__name__)  # 触发重试
async def create_task(request: ViduRequest, token: Optional[str] = None, vip: bool = False):
    token = token or await get_next_token_for_polling(
        FEISHU_URL_VIP if vip else FEISHU_URL,
        check_token=check_token,
        from_redis=True,
        min_points=4  # 最小消耗
    )

    token = token.strip(";Shunt=").strip("; Shunt=")

    task_type = TaskType.vidu_vip if vip else TaskType.vidu
    if vip:
        request.payload['settings']['schedule_mode'] = "nopeak"
        request.payload['settings']['resolution'] = "720p"

    headers = {
        "Cookie": token
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post("/tasks", json=request.payload)

        logger.debug(response.text)
        logger.debug(response.status_code)

        if response.status_code in {429, 403} or "insufficient" in response.text:  # 触发重试
            return True

        if response.is_success:
            data = response.json()
            task_id = f"{task_type}-{data['id']}"
            return Task(id=task_id, data=data, system_fingerprint=token)
        else:
            logger.debug(token)
            return Task(data=response.text, status=0, status_code=response.status_code)


@retrying(max_retries=8, max=8, predicate=lambda r: r is True, title=__name__)  # 触发重试
async def create_task_upscale(request: ViduUpscaleRequest, token: str):
    payload = {
        "input": {
            "creation_id": str(request.creation_id)
        },
        "type": "upscale",
        "settings": {"model": "stable", "duration": 4}  #######
    }
    headers = {
        "Cookie": token
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post("/tasks", json=payload)

        if response.status_code in {429} and "insufficient" in response.text:  # 触发重试
            return True

        if response.is_success:
            data = response.json()
            task_id = f"vidu-{data['id']}"
            return Task(id=task_id, data=data, system_fingerprint=token)
        else:
            return Task(data=response.text, status=0, status_code=response.status_code)


@retrying(max_retries=3, title=__name__)
async def get_task(task_id: str, token: str):  # https://api.vidu.studio/vidu/v1/tasks/state?id=2375528561004183
    task_id = task_id.split("-", 1)[-1]

    token = token.strip(";Shunt=").strip("; Shunt=")

    headers = {
        "Cookie": token
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.get(f"/tasks/{task_id}")

        logger.debug(response.text)
        logger.debug(response.status_code)

        if response.is_success:
            data = response.json()
            return data
        response.raise_for_status()


@retrying(title=__name__)
@alru_cache(ttl=10)
async def get_credits(token: str):
    headers = {
        "Cookie": token.rstrip(";Shunt=").strip("; Shunt="),
    }

    async with httpx.AsyncClient(headers=headers, timeout=60) as client:
        response = await client.get("https://api.vidu.studio/credit/v1/credits/me")
        logger.debug(response.status_code)
        logger.debug(response.text)

        if response.is_success:
            data = response.json()
            # {
            #     "credits": 1952,
            #     "credits_expire_today": 0,
            #     "credits_expire_monthly": 1936,
            #     "credits_permanent": 16
            # }
            return data
        response.raise_for_status()


async def check_token(token, threshold: float = 0):
    try:
        data = await get_credits(token)
        logger.debug(data['credits'])
        return data['credits'] > threshold  # 视频
    except Exception as e:
        logger.error(e)
        logger.debug(token)
        return False


if __name__ == '__main__':
    token = "io=Q07avSD7V1xT3PIAAZOO; HMACCOUNT_BFESS=5E705F99269C2211; Hm_lvt_a3c8711bce1795293b1793d35916c067=1737419786; Hm_lpvt_a3c8711bce1795293b1793d35916c067=1737419786; HMACCOUNT=5E705F99269C2211; NEXT_LOCALE=zh; _ga=GA1.1.1631944243.1737419791; sajssdk_2015_cross_new_user=1; JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3Mzg3MTU4MDQsImlhdCI6MTczNzQxOTgwNCwiaXNzIjoiaWFtIiwic3ViIjoiMjYxNDYwNzk0OTg5NjgyNSJ9.KmEoctBweMPmqSH-gTK00wHIb1iOz-UoMCf55zvhLfE; Shunt=; sensorsdata2015jssdkcross=dfm-enc-%7B%22Va28a6y8_aV%22%3A%22snESnARGSGtGntsH%22%2C%22gae28_aV%22%3A%22EGStnSisEnunHS-AAVrGEiGrSrIVnst-EuHsHnIR-GsEnAA-EGStnSisEnyERyG%22%2C%22OemO2%22%3A%7B%22%24ki8r28_8eiggay_2mbeyr_8cOr%22%3A%22%E7%9B%B4%E6%8E%A5%E6%B5%81%E9%87%8F%22%2C%22%24ki8r28_2rieyz_lrcMmeV%22%3A%22%E6%9C%AA%E5%8F%96%E5%88%B0%E5%80%BC_%E7%9B%B4%E6%8E%A5%E6%89%93%E5%BC%80%22%2C%22%24ki8r28_ergreere%22%3A%22%22%7D%2C%22aVr68a8ar2%22%3A%22rc3liZ7ku67OV5kgPsGCiskkDskl3qmawFlApXPAPF3JQo3sQFN8wXLlWFlJPFkkQ9KhWXPcpvAJPqKcQFPhQcAHwq1swX08wFlApXPAPF3JQowJQswH3aMax9klWZHAiD7HDsJCWskbDskl3qmawqPJQXPMQhlApFfHQqfcQ4xG%22%2C%22za28mec_kmfa6_aV%22%3A%7B%226ior%22%3A%22%24aVr68a8c_kmfa6_aV%22%2C%22Cikbr%22%3A%22snESnARGSGtGntsH%22%7D%7D; aws-waf-token=fb511419-d35f-4c57-a627-a0f362c1d04b:AAoAjVEDV+K9AAAA:ciqXmsweAEsZksfstr3LIrBKbKpvwnmWcdovZIcjh8DPtGXU4Fb5jxZMRBSIwrc7q9V0xJJZn9HW87Gkq9Y06CTKYUcExMKgu0RCm7Y+z7p6OERTrwY0+ACEWHW7B398/TTBTVW1+hjLkvO9DWrEJx4goQzZt+SFW/jZEITLszARozPYzeTEkvHJrIQkJLS0YbcxLRhaLtVQD+D0q++D8YFPsIbRgYRmeHpabAB9GbSV7vTKXREhdHyzj0QEkmmK6fyr; _ga_ZJBV7VYP55=GS1.1.1737419791.1.1.1737419806.45.0.0; SERVERCORSID=354f7c594b90555807f965c4af1d10d3|1737419812|1737419793"
    # FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=Hcr2i8"
    # arun(get_next_token_for_polling(FEISHU_URL, check_token=check_token))
    # arun(get_next_token_for_polling(FEISHU_URL_VIP, check_token=check_token, max_size=10000))

    # token = "_GRECAPTCHA=09AEXsBHm3ZGyFm6bsaF_HVQw8tBthjXeptYzdw7mGCczOlla0LlYHSb_HESN_xn3uXXR55VaHqXzc8EN2-0eCjQs; io=cmL_418SiY874dz6Ac4C; HMACCOUNT_BFESS=3B0B5E02E41153CB; Hm_lvt_a3c8711bce1795293b1793d35916c067=1725242991; Hm_lpvt_a3c8711bce1795293b1793d35916c067=1725242991; HMACCOUNT=3B0B5E02E41153CB; _ga=GA1.1.793646133.1725242994; sajssdk_2015_cross_new_user=1; JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MjY1MzkwMDAsImlhdCI6MTcyNTI0MzAwMCwiaXNzIjoiaWFtIiwic3ViIjoiMjQxNTEwMzE4NzgwNTkzMSJ9.AW77pOYME2EePiEV4TmtViBWPvPw_2tiY73R5s2APe8; Shunt=; sensorsdata2015jssdkcross=dfm-enc-%7B%22Va28a6y8_aV%22%3A%22sSEHEAIEtRtAHGIE%22%2C%22gae28_aV%22%3A%22EGEuARrrVIArig-AgnEgGiVVIyAyit-EuHsHnIR-GsEnAA-EGEuARrrVIEERSE%22%2C%22OemO2%22%3A%7B%22%24ki8r28_8eiggay_2mbeyr_8cOr%22%3A%22%E7%9B%B4%E6%8E%A5%E6%B5%81%E9%87%8F%22%2C%22%24ki8r28_2rieyz_lrcMmeV%22%3A%22%E6%9C%AA%E5%8F%96%E5%88%B0%E5%80%BC_%E7%9B%B4%E6%8E%A5%E6%89%93%E5%BC%80%22%2C%22%24ki8r28_ergreere%22%3A%22%22%7D%2C%22aVr68a8ar2%22%3A%22rc3liZ7ku67OV5kgPsGCiskkDskl3qmawFlJPq0IWZdlwhLkPZP8w9PswZPHPZ7lwswMPs1SBFTaQF3EQqwIBFlcwFPMwvAJpFTawXVkWZNhwF1IQX1aBv3liZ7ku67OV5kgu9G6iZHgiZNapa3cQX1EwF0hwFfIpX0EpFwJ36A%3D%22%2C%22za28mec_kmfa6_aV%22%3A%7B%226ior%22%3A%22%24aVr68a8c_kmfa6_aV%22%2C%22Cikbr%22%3A%22sSEHEAIEtRtAHGIE%22%7D%7D; SERVERCORSID=75eeb9319017a9014e79152a283d6cbe|1725243001|1725242994; _ga_ZJBV7VYP55=GS1.1.1725242993.1.1.1725243001.0.0.0"
    #
    # token = '_GRECAPTCHA=09AA5Y-DKEaPCPtfl_s0o9z-HKEP5Tkfrn7CsmZfUj5MUYAFZiW7ELincbr2c2baFkM5Vu_KDPJ11l_N_DJhHTx_A; HMACCOUNT_BFESS=4189E6AE98589913; Hm_lvt_a3c8711bce1795293b1793d35916c067=1722407159; Hm_lpvt_a3c8711bce1795293b1793d35916c067=1722407159; HMACCOUNT=4189E6AE98589913; io=FKh7a-dvlK-UnBmOAB7M; JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MjM3MDMxNjgsImlhdCI6MTcyMjQwNzE2OCwiaXNzIjoiaWFtIiwic3ViIjoiMjM2ODY0MDkxNDEwMTMxOCJ9.mbNkzI6piihIVDsGSshSCiLlakAAG_Hxh0xAnPkx_vs; Shunt=; Hm_lvt_a3c8711bce1795293b1793d35916c067=1753943158594|1722407159; shortid=parsvjrji; debug=undefined; _grecaptcha=09AA5Y-DJauC2Mo_KPc5_R5OUPR__wsqLYpOjUViIajU8hpDPOpg6LcH1xECbelcZwXl_BSHnZlCWRGxGWvRvMyQr7QGnCCJam75DKvw; VIDU_SELECTED_LOCALE=zh; VIDU_TOUR="v1"'
    # token = "JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MjM3MDIyNjAsImlhdCI6MTcyMjQwNjI2MCwiaXNzIjoiaWFtIiwic3ViIjoiMjM2ODYyNjA0MzE4MTg1NCJ9.8P8WuktyjFM0utFbGcxLQptzfv43ugmMcE31RXR9JJQ"
    file = Path('/Users/betterme/PycharmProjects/AI/cover.jpeg').read_bytes()
    # url = arun(upload(file, vip=True)).url

    # print(arun(get_next_token_for_polling(FEISHU_URL)) == token)

    # arun(get_task("vidu-2368380300283813", token=token))
    # arun(get_task('vidu-2389570992698888', token=token))

    # arun(create_task_upscale())

    # token="_ga=GA1.1.2058758439.1724312077; Shunt=; JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3Mjk5MTIxNDcsImlhdCI6MTcyODYxNjE0NywiaXNzIjoiaWFtIiwic3ViIjoiMjQyNzk2NjczODQwNjY3OSJ9.VFLEBmnL8_Biy_46X19Fr5PkbwxdLXx3Xq2GagsW2Fw; sensorsdata2015jssdkcross=dfm-enc-%7B%22Va28a6y8_aV%22%3A%22sSsRGnnRItSAnnRG%22%2C%22gae28_aV%22%3A%22EGEuAnststSEirt-ARSAigSVHIiHVs-EtHsHnIR-sARInAA-EGEuAnststHsIti%22%2C%22OemO2%22%3A%7B%22%24ki8r28_8eiggay_2mbeyr_8cOr%22%3A%22%E8%87%AA%E7%84%B6%E6%90%9C%E7%B4%A2%E6%B5%81%E9%87%8F%22%2C%22%24ki8r28_2rieyz_lrcMmeV%22%3A%22%E6%9C%AA%E5%8F%96%E5%88%B0%E5%80%BC%22%2C%22%24ki8r28_ergreere%22%3A%22z88O2%3A%2F%2Fiyymb682.fmmfkr.ymo%2F%22%7D%2C%22aVr68a8ar2%22%3A%22rc3liZ7ku67OV5kgPsGCiskkDskl3qmawFlJPq0swqfcpXNJPZKSBF0IQXLzWq7lQFQzQZNcBF1SQF3EQqwIBF3MQhwswX08wFlJPq0swqfcpXKcwhzz3aMax9klWZHAiD7HDsJCWskbDskl3qmawqNcQhlsQqyhpXNMQqPIp4xG%22%2C%22za28mec_kmfa6_aV%22%3A%7B%226ior%22%3A%22%24aVr68a8c_kmfa6_aV%22%2C%22Cikbr%22%3A%22sSsRGnnRItSAnnRG%22%7D%7D; _ga_ZJBV7VYP55=GS1.1.1728614362.54.1.1728616189.0.0.0"
    # arun(get_credits(token))
    #
    d = {
        "model": 'vidu-2.0',
        "prompt": "这个女人笑起来",
        "url": "https://oss.ffire.cc/files/kling_watermark.png",  # failed to save uploads
        "tail_image_url": "https://oss.ffire.cc/files/kling_watermark.png",
    }
    token = None
    # print(bjson(ViduRequest(**d).payload))
    # arun(create_task(ViduRequest(**d), vip=True))
    arun(create_task(ViduRequest(**d), vip=False))
    # # pass
    # token = "sensorsdata2015jssdkcross=dfm-enc-%7B%22Va28a6y8_aV%22%3A%22sSsAAnHAInAGEtIG%22%2C%22gae28_aV%22%3A%22EGEuAnststSEirt-ARSAigSVHIiHVs-EtHsHnIR-sARInAA-EGEuAnststHsIti%22%2C%22OemO2%22%3A%7B%22%24ki8r28_8eiggay_2mbeyr_8cOr%22%3A%22%E7%9B%B4%E6%8E%A5%E6%B5%81%E9%87%8F%22%2C%22%24ki8r28_2rieyz_lrcMmeV%22%3A%22%E6%9C%AA%E5%8F%96%E5%88%B0%E5%80%BC_%E7%9B%B4%E6%8E%A5%E6%89%93%E5%BC%80%22%2C%22%24ki8r28_ergreere%22%3A%22%22%7D%2C%22aVr68a8ar2%22%3A%22rc3liZ7ku67OV5kgPsGCiskkDskl3qmawFlJPq0swqfcpXNJPZKSBF0IQXLzWq7lQFQzQZNcBF1SQF3EQqwIBF3MQhwswX08wFlJPq0swqfcpXKcwhzz3aMax9klWZHAiD7HDsJCWskbDskl3qmawqNcwX0sQF0hQq0HwFfhp4xG%22%2C%22za28mec_kmfa6_aV%22%3A%7B%226ior%22%3A%22%24aVr68a8c_kmfa6_aV%22%2C%22Cikbr%22%3A%22sSsAAnHAInAGEtIG%22%7D%7D;_ga=GA1.1.2058758439.1724312077;_ga_ZJBV7VYP55=GS1.1.1727080335.38.1.1727080510.0.0.0;JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MjgzNTAzODAsImlhdCI6MTcyNzA1NDM4MCwiaXNzIjoiaWFtIiwic3ViIjoiMjQyMDA2NTAzNjA5MTgzOSJ9.PkjQqjYB56vYetYwmlagnWn_6bSCwoxCjI7BjfelBOU;Shunt="
    # # token = "_ga=GA1.1.2058758439.1724312077; JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MjgzNTAzODAsImlhdCI6MTcyNzA1NDM4MCwiaXNzIjoiaWFtIiwic3ViIjoiMjQyMDA2NTAzNjA5MTgzOSJ9.PkjQqjYB56vYetYwmlagnWn_6bSCwoxCjI7BjfelBOU; Shunt=; sensorsdata2015jssdkcross=dfm-enc-%7B%22Va28a6y8_aV%22%3A%22sSsAAnHAInAGEtIG%22%2C%22gae28_aV%22%3A%22EGEuAnststSEirt-ARSAigSVHIiHVs-EtHsHnIR-sARInAA-EGEuAnststHsIti%22%2C%22OemO2%22%3A%7B%22%24ki8r28_8eiggay_2mbeyr_8cOr%22%3A%22%E7%9B%B4%E6%8E%A5%E6%B5%81%E9%87%8F%22%2C%22%24ki8r28_2rieyz_lrcMmeV%22%3A%22%E6%9C%AA%E5%8F%96%E5%88%B0%E5%80%BC_%E7%9B%B4%E6%8E%A5%E6%89%93%E5%BC%80%22%2C%22%24ki8r28_ergreere%22%3A%22%22%7D%2C%22aVr68a8ar2%22%3A%22rc3liZ7ku67OV5kgPsGCiskkDskl3qmawFlJPq0swqfcpXNJPZKSBF0IQXLzWq7lQFQzQZNcBF1SQF3EQqwIBF3MQhwswX08wFlJPq0swqfcpXKcwhzz3aMax9klWZHAiD7HDsJCWskbDskl3qmawqNcwX0sQF0hQq0HwFfhp4xG%22%2C%22za28mec_kmfa6_aV%22%3A%7B%226ior%22%3A%22%24aVr68a8c_kmfa6_aV%22%2C%22Cikbr%22%3A%22sSsAAnHAInAGEtIG%22%7D%7D; _ga_ZJBV7VYP55=GS1.1.1727080335.38.1.1727080510.0.0.0"
    # token = "JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MjgzNDkyNzQsImlhdCI6MTcyNzA1MzI3NCwiaXNzIjoiaWFtIiwic3ViIjoiMjQyMDg5NjA4MTIwNTkwNyJ9.MRXmSr48PifQgRN1-yTTu8d7Sq1An4OS7G5WoYpJ_PU"

    # token = "_ga=GA1.1.1191408146.1725443726; Shunt=; JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzY0NzgzODIsImlhdCI6MTczNTE4MjM4MiwiaXNzIjoiaWFtIiwic3ViIjoiMjU3NTg1MzI5MDUyNzYwNiJ9.AJBuQH0B9aQ2znnczYoqBw9YvHkLh0uZF0QQx5xdxrc; sensorsdata2015jssdkcross=dfm-enc-%7B%22Va28a6y8_aV%22%3A%22sHRHtHIsGAHsRnAn%22%2C%22gae28_aV%22%3A%22EGEuyRHVigIEtAG-AGRuSsHrVAGRuS-EtHsHnIR-sARInAA-EGEuyRHVigSEuIR%22%2C%22OemO2%22%3A%7B%22%24ki8r28_8eiggay_2mbeyr_8cOr%22%3A%22%E8%87%AA%E7%84%B6%E6%90%9C%E7%B4%A2%E6%B5%81%E9%87%8F%22%2C%22%24ki8r28_2rieyz_lrcMmeV%22%3A%22%E6%9C%AA%E5%8F%96%E5%88%B0%E5%80%BC%22%2C%22%24ki8r28_ergreere%22%3A%22z88O2%3A%2F%2Fiyymb682.fmmfkr.ymo%2F%22%7D%2C%22aVr68a8ar2%22%3A%22rc3liZ7ku67OV5kgPsGCiskkDskl3qmawFlJPowIQZ7zWqwJpX0HBF0HQs3AwqdkWX0HQs3ABF1SQF3EQqwIBF3MQhwswX08wFlJPowIQZ7zWqNJPqwI3aMax9klWZHAiD7HDsJCWskbDskl3qmawqKIQFfEwh3HwXKcQhPMQaxG%22%2C%22za28mec_kmfa6_aV%22%3A%7B%226ior%22%3A%22%24aVr68a8c_kmfa6_aV%22%2C%22Cikbr%22%3A%22sHRHtHIsGAHsRnAn%22%7D%7D; _ga_ZJBV7VYP55=GS1.1.1735182278.4.1.1735182405.26.0.0"
    # arun(check_token(token))
    #
    # FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=rxldsA"
    # FEISHU_URL="https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=Hcr2i8"
    # api_keys = get_spreadsheet_values(feishu_url=FEISHU_URL, to_dataframe=True)[0]
    #
    #
    # for i in tqdm(api_keys):
    #     logger.debug(i)
    #     if i and arun(check_token(i)):
    #         j = i
    #     # for j in (i or '').split(';'):
    #     #     if j.startswith('JWT='):
    #         with open('xx.txt', 'a') as f:
    #             f.write(j + '\n')
    # #
    # FEISHU_URL_VIP
    # arun(get_next_token_for_polling(FEISHU_URL_VIP,
    #                                 check_token=check_token,
    #                                 from_redis=True))

    # arun(get_next_token_for_polling(FEISHU_URL,
    #                                 check_token=check_token,
    #                                 from_redis=True))

    # from meutils.db.redis_db import redis_aclient

    # arun(redis_aclient.delete(FEISHU_URL))

    # token = "_ga=GA1.1.633410699.1728736789; Shunt=; sensorsdata2015jssdkcross=dfm-enc-%7B%22Va28a6y8_aV%22%3A%22sSRAsSHEntRnsHns%22%2C%22gae28_aV%22%3A%22EGstAuVggRArIr-AtnSIGARtIVIrSt-EnHsHnIR-EEtRAHn-EGstAuVggRErEt%22%2C%22OemO2%22%3A%7B%22%24ki8r28_8eiggay_2mbeyr_8cOr%22%3A%22%E7%9B%B4%E6%8E%A5%E6%B5%81%E9%87%8F%22%2C%22%24ki8r28_2rieyz_lrcMmeV%22%3A%22%E6%9C%AA%E5%8F%96%E5%88%B0%E5%80%BC_%E7%9B%B4%E6%8E%A5%E6%89%93%E5%BC%80%22%2C%22%24ki8r28_ergreere%22%3A%22%22%7D%2C%22aVr68a8ar2%22%3A%22rc3liZ7ku67OV5kgPsGCiskkDskl3qmawFlcpXLaW9WoQhLkwsK8wXfsQXwHwXySwsNhWFNSBF1sQF3EQqwIBF1JpXyMQFP8wFlcpXLaW9WoQhTkwFfaBv3liZ7ku67OV5kgu9G6iZHgiZNapa3cQXyMwqNEwFPSQhPcQFPc36A%3D%22%2C%22za28mec_kmfa6_aV%22%3A%7B%226ior%22%3A%22%24aVr68a8c_kmfa6_aV%22%2C%22Cikbr%22%3A%22sSRAsSHEntRnsHns%22%7D%7D; JWT=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzIwMDM4NzAsImlhdCI6MTczMDcwNzg3MCwiaXNzIjoiaWFtIiwic3ViIjoiMjQ3MDI0NTE2ODc2MjU2MiJ9.uK3-CHB_6th4hha_IMI56KT7OLfwq8m-rOJCjwo0sxM; _ga_ZJBV7VYP55=GS1.1.1730707682.2.1.1730707881.0.0.0"
    #
    # arun(check_token(token))



# -*- coding: utf-8 -*-

# Python 2.7 workaround
try:
    import pathlib
except ImportError:
    pass


_NUM_SIGNATURE_BYTES = 8192


def get_signature_bytes(path):
    """
    Reads file from disk and returns the first 8192 bytes
    of data representing the magic number header signature.

    Args:
        path: path string to file.

    Returns:
        First 8192 bytes of the file content as bytearray type.
    """
    with open(path, 'rb') as fp:
        return bytearray(fp.read(_NUM_SIGNATURE_BYTES))


def signature(array):
    """
    Returns the first 8192 bytes of the given bytearray
    as part of the file header signature.

    Args:
        array: bytearray to extract the header signature.

    Returns:
        First 8192 bytes of the file content as bytearray type.
    """
    length = len(array)
    index = _NUM_SIGNATURE_BYTES if length > _NUM_SIGNATURE_BYTES else length

    return array[:index]


def get_bytes(obj):
    """
    Infers the input type and reads the first 8192 bytes,
    returning a sliced bytearray.

    Args:
        obj: path to readable, file-like object(with read() method), bytes,
        bytearray or memoryview

    Returns:
        First 8192 bytes of the file content as bytearray type.

    Raises:
        TypeError: if obj is not a supported type.
    """
    if isinstance(obj, bytearray):
        return signature(obj)

    if isinstance(obj, str):
        return get_signature_bytes(obj)

    if isinstance(obj, bytes):
        return signature(obj)

    if isinstance(obj, memoryview):
        return bytearray(signature(obj).tolist())

    if isinstance(obj, pathlib.PurePath):
        return get_signature_bytes(obj)

    if hasattr(obj, 'read'):
        if hasattr(obj, 'tell') and hasattr(obj, 'seek'):
            start_pos = obj.tell()
            obj.seek(0)
            magic_bytes = obj.read(_NUM_SIGNATURE_BYTES)
            obj.seek(start_pos)
            return get_bytes(magic_bytes)
        return get_bytes(obj.read(_NUM_SIGNATURE_BYTES))

    raise TypeError('Unsupported type as file input: %s' % type(obj))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2024/7/5 12:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo
# https://docs.fish.audio/api-reference/endpoint/openapi-v1/speech-to-text
# https://docs.fish.audio/text-to-speech/create-model
import httpx

from meutils.pipe import *
from meutils.schemas.task_types import Task
from meutils.schemas.openai_types import TTSRequest
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.notice.feishu import send_message as _send_message

from openai.types.file_object import FileObject
from fastapi import UploadFile

BASE_URL = "https://api.fish.audio"
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/ZUyLsVyjMhtexRt3IB9ci9fUnWc?sheet=7ce4e3"

url = "https://api.fish.audio/model"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/f0475882-ad39-49d9-ad77-523c0e768e96"
)


async def check_token(token: str, for_api: bool = False):
    path = "api-credit" if for_api else "package"
    try:
        headers = {
            "Authorization": f"Bearer {token}"
        }
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers) as client:
            response = await client.get(f"/wallet/self/{path}")
            response.raise_for_status()
            data = response.json()

            # logger.debug(data)

            credit = data.get('credit') or data.get('balance', 0)
            logger.info(credit)

            return float(credit) > 0
    except Exception as e:
        logger.error(e)
        return False


async def get_model_list(token, **kwargs):  # 上传的音色
    params = {
        'self': True,
        'title': None,

        'page_size': 10,
        'page_number': 1,
        **kwargs
    }
    headers = {
        "authorization": f"Bearer {token}"
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers) as client:
        response = await client.get("/model", params=params)
        return response.is_success and response.json()


async def create_task(request: TTSRequest):
    """
        await create_task(request, stream=True)
    :param request:
    :return:
    """
    token = await get_next_token_for_polling(feishu_url=FEISHU_URL)

    headers = {
        "authorization": f"Bearer {token}"
    }
    payload = {
        "type": "tts",
        "channel": "free",
        "stream": True,  # 区别？
        "model": request.voice,
        "parameters": {
            "text": request.input
        }
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.post("/task", json=payload)
        if response.is_success:
            logger.debug(response.headers)

            task_id = response.headers.get("task-id")
            task_response = await client.get(f"/task/{task_id}")
            data = task_response.json()
            data['url'] = data['result']
            data['fileview'] = url2fileview(data['result'])
            return data

        response.raise_for_status()


async def create_file_for_openai(file: UploadFile):  # todo: 存储 redis
    token = await get_next_token_for_polling(feishu_url=FEISHU_URL)

    filename = file.filename or file.file.name
    model_info = await create_tts_model(token, file=(filename, file.file))

    if isinstance(model_info, dict):
        model_id = model_info.get("_id")
        file_id = model_id
        status = "processed"
    else:
        file_id = shortuuid.random()
        status = "error"

    file_object = FileObject.construct(
        id=file_id,

        filename=filename,  # result.get("file_name")
        bytes=file.size,

        created_at=int(time.time()),
        object='file',

        purpose="voice-clone",
        status=status,

        data=model_info
    )
    return file_object


async def create_tts_model(token, file, **kwargs):
    """
    {'_id': '9d10cdbea3954aa9b8fd992fd24b92a7',
     'author': {'_id': 'd71d7c63c52e4d70be72e3afdb7952ab',
                'avatar': '',
                'nickname': '313303303'},
     'cover_image': 'coverimage/9d10cdbea3954aa9b8fd992fd24b92a7',
     'created_at': '2024-07-05T05:30:32.020433Z',
     'description': '',
     'languages': ['zh'],
     'like_count': 0,
     'liked': False,
     'mark_count': 0,
     'marked': False,
     'samples_text': [],
     'shared_count': 0,
     'state': 'trained',
     'tags': [],
     'task_count': 0,
     'title': 'chatfire',
     'train_mode': 'fast',
     'type': 'tts',
     'updated_at': '2024-07-05T05:30:32.020407Z',
     'visibility': 'public'}
    """
    payload = {
        **kwargs
    }

    logger.debug(file)

    files = {
        'title': (None, 'chatfire-tts'),
        'description': (None, ''),
        'type': (None, 'tts'),
        'train_mode': (None, 'fast'),
        'visibility': (None, 'public'),  # private

        'voices': file,  # ('audio_name.mp3', file)
        'cover_image': open(get_resolve_path('cowboy-hat-face.webp', __file__), 'rb')

    }

    headers = {
        "authorization": f"Bearer {token}"
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.post("/model", json=payload, files=files)
        # logger.debug(response.text)
        if response.is_success:
            _ = response.json()
            _['model_id'] = _['_id']
            return _
        else:
            return response.text


if __name__ == '__main__':
    token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VyX2lkIjoiZDcxZDdjNjNjNTJlNGQ3MGJlNzJlM2FmZGI3OTUyYWIiLCJleHAiOjE3MzAyODYwNjguNTMyMTA5NX0.GNeW2WLlBqyP6evNQFkXpqa0UYOg50_IwNO48DuJc04"
    # print(bjson(arun(get_model_list(token))))
    # file = open('/Users/betterme/Downloads/whisper-1719913495729-54f08dde5.wav.mp3.mp3', 'rb')

    # arun(get_model_list(token))
    # file = UploadFile(file)
    # pprint(arun(create_tts_model(token, file=(file.file.name, file.file))))
    # model = "9d10cdbea3954aa9b8fd992fd24b92a7"
    # # # task_id = "f5a94a3fb78646b8ab1c3606413ca9e0"
    # with timer():
    #     text = "文本转语音\n文本转语音\n文本转语音\n文本转语音\n"
    #     request = TTSRequest(input=text, model=model)
    #     arun(create_task(request))
    # open("xx.mp3", 'wb').write(f)

    # async def main():
    #     content = await create_task(request, stream=True)
    #     with open("xx.mp3", 'wb') as f:
    #         f.write(content)

    # arun(main())

    # pprint(arun(get_tts_task(token, task_id)))

    #
    # file = open('/Users/betterme/Downloads/whisper-1719913495729-54f08dde5.wav.mp3.mp3', 'rb')
    #
    # file = UploadFile(file)
    #
    # print(arun(upload_audio_for_tts_model(file=file)))

    # token = "5cf445234491429c9855d3fa666d49f6"

    # arun(check_token(token, for_api=True))

    #     text = "文本转语音\n文本转语音\n文本转语音\n文本转语音\n"


    arun(create_task(TTSRequest(input="你好", model="")))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2023/8/21 14:26
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

os.environ["EVN_FILE"] = "/Users/betterme/PycharmProjects/AI/aiapi/aiapi/core/dev.env"

from meutils.settings.base import BaseSetting


class TestSetting(BaseSetting):
    name: str = '初始化'


print(TestSetting())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : oneapi
# @Time         : 2024/6/28 09:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/Thekers/Get_OpenaiKey/blob/9d174669d7778ea32d1132bedd5167597912dcfb/Add_01AI_Token.py
import jsonpath
from meutils.pipe import *
from meutils.schemas.oneapi import REDIRECT_MODEL, NOTICE, FOOTER
from meutils.schemas.oneapi import MODEL_PRICE, MODEL_RATIO, COMPLETION_RATIO, GROUP_RATIO
from meutils.schemas.oneapi.model_info import model_info
from meutils.schemas.oneapi.model_group_info import model_group_info

import requests
import json


# 500000对应1块
def option(token: Optional[str] = None):
    token = token or os.environ.get("CHATFIRE_ONEAPI_TOKEN")

    url = "https://api.chatfire.cn/api/option/"
    headers = {
        'authorization': f'Bearer {token}',
        'rix-api-user': '1'
    }

    payloads = [
        # {
        #     "key": "Notice",
        #     "value": NOTICE
        # },

        {
            "key": "Footer",
            "value": FOOTER
        },

        {
            "key": "ModelPrice",
            "value": json.dumps(MODEL_PRICE)
        },
        {
            "key": "ModelRatio",
            "value": json.dumps(MODEL_RATIO)
        },
        {
            "key": "CompletionRatio",
            "value": json.dumps(COMPLETION_RATIO)
        },
        # {
        #     "key": "GroupRatio",
        #     "value": json.dumps(GROUP_RATIO)
        # },
        # {
        #     "key": "UserUsableGroups",
        #     "value": json.dumps(USER_USABLE_GROUPS)
        # },

        # 模型信息
        {
            "key": "ModelInfo",
            "value": json.dumps(model_info, indent=4)
        },
        # 厂商信息
        {
            "key": "ModelGroupInfo",
            "value": json.dumps(model_group_info, indent=4)
        },
    ]
    for payload in payloads:
        response = requests.request("PUT", url, headers=headers, json=payload)
        logger.debug(response.json())


def add_channel(
        base_url,
        api_keys: list,
        models: list,
        url: Optional[str] = None,
        token: Optional[str] = None,
):
    token = token or os.environ.get("CHATFIRE_ONEAPI_TOKEN")

    url = url or "https://api.chatfire.cn/api/channel"
    headers = {
        'authorization': f'Bearer {token}',
        'rix-api-user': '1'
    }

    payload = {

        "name": base_url,  # 渠道名

        "type": 1,
        "base_url": base_url,
        "key": '\n'.join(api_keys),
        "models": ','.join(models),

        "openai_organization": "",
        "max_input_tokens": 0,
        "other": "",
        "model_mapping": "",
        "status_code_mapping": "",
        "auto_ban": 1,
        "test_model": "",
        "groups": [
            "default"
        ],
        "group": "default"
    }
    response = requests.request("POST", url, headers=headers, json=payload)
    logger.debug(response.text)

    return response.json()


# todo: 同一个任务补偿一次【加个缓存】，再想想【redis加个标识，判断吧】
# def compensate_user_money(callback_fn):
#     if set(jsonpath.jsonpath(data, "$.status")) == {"error"}:  # 触发补偿，如何去重【剔除id？】
#         pass

# https://api.chatfire.cn/api/log/token?key=sk-
if __name__ == '__main__':
    option()
    # print(json.dumps(MODEL_RATIO, indent='\n'))
    # print(add_channel(
    #     base_url="xx",
    #     api_keys=['xx'],
    #     models=['xx'],
    #     token=""
    # ))



#
# from .chinese_text_splitter import *
#
#
#
# from langchain.text_splitter import RecursiveCharacterTextSplitter
#
#
from langchain.text_splitter import RecursiveCharacterTextSplitter
textsplitter = RecursiveCharacterTextSplitter(separators=["。"], chunk_overlap=10, chunk_size=100)



from meutils.pipe import *
from meutils.decorators.retry import retrying

from playwright.sync_api import Playwright, sync_playwright


def my_request(request):
    """https://blog.csdn.net/B11050729/article/details/131293769
    """
    print(request.all_headers())


@retrying(predicate=lambda r: r and len(r['data']['chatDetailList'][0]['data']) < 5)
def response_parser(url):
    r = requests.get(url).json()
    # logger.debug(len(r['data']['chatDetailList'][0]['data']))
    return r


def run(playwright: Playwright) -> None:
    iphone = {}
    # iphone = playwright.devices['iPhone 15']

    browser = playwright.chromium.launch(headless=False)
    context = browser.new_context(**iphone)
    page = context.new_page()

    page.on("request", my_request)
    page.goto("https://search.tiangong.cn/")
    logger.debug(page.url)

    page.get_by_placeholder("试试输入任何想了解的问题").fill("林俊杰")
    page.wait_for_load_state(state='networkidle')

    page.get_by_placeholder("试试输入任何想了解的问题").press("Enter")

    page.wait_for_load_state(state='networkidle')
    page.wait_for_url(page.url, wait_until='networkidle')

    page.wait_for_timeout(30 * 1000)
    logger.debug(page.url)

    page.screenshot(full_page=True, path='screenshot.png')
    page.pdf(path='p.pdf')
    conversationId = page.url.split('/')[-1]
    url = f"https://api-search.tiangong.cn/chat-user/api/chat/conversation/detail?conversationId={conversationId}"

    logger.debug(url)

    response = response_parser(url)
    response['data']['url'] = url

    #
    # new_page = new_page_info.value
    # new_page.wait_for_load_state()
    # print(new_page)
    #
    # page.screenshot(full_page=True, path='screenshot.png')
    # # print(page)
    # page.wait_for_load_state('networkidle')
    # page.wait_for_timeout(5000)
    #
    # page.pdf(path='p.pdf')

    # "//*[@id="app"]/div[1]/div[1]/div[1]/div/div/div/div[1]/div/div[2]"
    # "//*[@id="app"]/div[1]/div[1]/div[1]/div/div/div/div[1]/div/div[3]"

    #
    # page.locator("div").filter(has_text=re.compile(r"^参考$")).click()
    # with page.expect_popup() as page1_info:
    #     page.get_by_text("参考郭富城赛车时发生车祸：车辆故障被迫退赛 - 快科技mydrivers · 1郭富城赛车时发生车祸！回应来了 - 每日经济新闻nbd · 2郭富城赛车时发生车").click()
    # page1 = page1_info.value
    # page1.close()
    # page.locator(".el-scrollbar__view > div").click()
    #
    # with page.expect_popup() as page4_info:
    #     page.locator("#md-editor-v3-preview-wrapper").click()
    # page4 = page4_info.value
    # page4.close()
    # page.locator(".el-scrollbar__view > div").click()
    # with page.expect_popup() as page5_info:
    #     page.locator("#md-editor-v3-preview-wrapper").click()
    # page5 = page5_info.value
    # page5.locator("html").click(modifiers=["Meta"])
    # page5.close()

    # ---------------------
    context.close()
    browser.close()


with sync_playwright() as playwright:
    run(playwright)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : express
# @Time         : 2024/4/24 17:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

@lru_cache
def express_query(express_no=780098068058, path: str = '/kdi', appcode: Optional[str] = None):
    """

    :param appcode:
        appcode = '0ccd86184de94ca19c37cbb215b1f3722'  # 开通服务后 买家中心-查看AppCode

    :param express_no:
    :param path:
    :return:
    """
    base_url = "https://wuliu.market.alicloudapi.com"
    with httpx.Client(base_url=base_url) as client:
        headers = {"Authorization": f'APPCODE {appcode or os.getenv("EXPRESS_APP_CODE")}'}
        params = {'no': express_no}

        response = client.get(path, params=params, headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            return {"error": response.text}


async def aexpress_query(express_no=780098068058, path: str = '/kdi', appcode: Optional[str] = None):
    """

    :param appcode:
        appcode = '0ccd86184de94ca19c37cbb215b1f372'  # 开通服务后 买家中心-查看AppCode

    :param express_no:
    :param path:
    :return:
    """
    base_url = "https://wuliu.market.alicloudapi.com"
    async with httpx.AsyncClient(base_url=base_url) as client:
        headers = {"Authorization": f'APPCODE {appcode or os.getenv("EXPRESS_APP_CODE")}'}
        params = {'no': express_no}
        response = await client.get(path, params=params, headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            return {"error": response.text}


if __name__ == '__main__':
    # print(express_query('0ccd86184de94ca19c37cbb215b1f372'))

    print(arun(aexpress_query('0ccd86184de94ca19c37cbb215b1f372')))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : 1_分词
# @Time         : 2023/5/21 17:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


import streamlit as st
from annotated_text import annotated_text, annotation

from meutils.pipe import *
from meutils.ai_nlp.ner import Ner
from meutils.serving.streamlit import *

title = Path(__file__).stem.split('_')[-1]
st.set_page_config(title, page_icon='🔥', layout='wide', initial_sidebar_state='collapsed')
st.title(title)

hide_st_style('🔥DevelopedBy@数据科学研发中心-AI团队')

with st.form(title):
    text = """
    东北证券股份有限公司（以下简称“公司”）前身为吉林省证券有限责任公司。2000 年 6 月经中国证监会批准，经过增资扩股成立东北证券有限责任公司。
    """
    container = st.container()
    cols = st.columns(3)
    with cols[0]:
        options = ('快速模式', '精确模式')
        option2mode = dict(zip(options, ['fast', 'accurate']))
        options = st.multiselect('算法：', options=options, default=options)

    with cols[1]:
        is_filter = st.selectbox('仅保留实体：', ('否', '是')) == '是'

    text = st.text_area("文本", text.strip(), height=100)
    if st.form_submit_button('开始切词'):
        for option in options:
            nlp = Ner(mode=option2mode.get(option), entity_only=is_filter)
            _  =nlp([text])[0]
            annotations = []
            for word, flag in _:
                annotations.append(annotation(word, label=flag, color="blue"))
            st.markdown(f'\n')
            st.markdown(option+':')
            annotated_text(*annotations)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : headers
# @Time         : 2025/2/23 00:20
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from fastapi import FastAPI, Request, Depends, HTTPException

# from meutils.pipe import *
from meutils.str_utils.json_utils import repair_json


def get_headers(request: Request):
    dic = dict(request.headers)
    _dic = {k.replace('-', '_'): v for k, v in dic.items()}  # 增强兼容性

    if x_headers := dic.get('x-headers'):  # -H 'x-headers: {a: 1}' todo
        dic['x-headers'] = repair_json(x_headers, return_objects=True)

        # logger.debug(dic['x-headers'])
    if x_model_mapper := dic.get('x-model-mapper'):  #
        dic['x-model-mapper'] = eval(f"lambda r: {x_model_mapper}")

    return {**dic, **_dic}


if __name__ == '__main__':
    def get_headers():
        dic = {
            "x-headers": "{a:1}",

            'x-model-mapper': "r.get('a')"
        }
        # upstream_base_url = headers.get('upstream-base-url')
        if x_headers := dic.get('x-headers'):  # -H 'x-headers: {a: 1}' todo
            dic['x-headers'] = repair_json(x_headers, return_objects=True)

        if x_model_mapper := dic.get('x-model-mapper'):  #
            dic['x-model-mapper'] = eval(f"lambda r: {x_model_mapper}")

        return dic


    print(get_headers())




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : image_enhance
# @Time         : 2025/7/4 18:15
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import httpx

from meutils.pipe import *
from meutils.caches import rcache

from meutils.apis.utils import make_request
from meutils.io.files_utils import to_base64, to_url


@rcache(ttl=1 * 24 * 3600)
async def get_baidubce_access_token(token: Optional[str] = None):
    token = token or 'Uj2Q0X4YLGUl7jEj71evIkP2|QRAknDeIKU87ULXHDsf5RQej30Nge4pr'

    ak, sk = token.split('|')
    response = await make_request(
        base_url="https://aip.baidubce.com",
        path="/oauth/2.0/token",
        params={
            "grant_type": "client_credentials",
            "client_id": ak,
            "client_secret": sk
        },
        method="GET"
    )
    return response.get('access_token')


@rcache(ttl=1 * 24 * 3600)
async def image_enhance(image, response_format: str = "url"):
    access_token = await get_baidubce_access_token()

    params = {
        "access_token": access_token
    }
    base_url = "https://aip.baidubce.com"
    path = "/rest/2.0/image-process/v1/image_definition_enhance"

    if image.startswith('http'):
        payload = {'url': image}
    else:
        payload = {'image': await to_base64(image, content_type='image/png')}

    async with httpx.AsyncClient(base_url=base_url, params=params) as client:
        response = await client.post(path, data=payload)
        response.raise_for_status()
        response = response.json()

        if response_format == "url" and response.get('image'):
            response['image'] = await to_url(response['image'], content_type='image/png')

        return response

async def image_enhance_(image, response_format: str = "url"):
    access_token = await get_baidubce_access_token()

    params = {
        "access_token": access_token
    }
    base_url = "https://aip.baidubce.com"
    path = "/rest/2.0/image-process/v1/image_definition_enhance"

    if image.startswith('http'):
        payload = {'url': image}
    else:
        payload = {'image': await to_base64(image, content_type='image/png')}

    async with httpx.AsyncClient(base_url=base_url, params=params) as client:
        response = await client.post(path, data=payload)
        response.raise_for_status()
        response = response.json()

        if response_format == "url" and response.get('image', '').startswith('data:image'):
            response['image'] = await to_url(response['image'], content_type='image/png')

        return response


if __name__ == '__main__':
    image = "https://oss.ffire.cc/files/liu.jpg"
    image = "https://jinhua-1359405894.cos.ap-shanghai.myqcloud.com/mj/2025/06/23/3bc3df77-f413-4a78-a191-31ec5a014ddd.png"
    _ = arun(image_enhance(image))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2024/4/24 14:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://python.langchain.com/docs/modules/model_io/output_parsers/types/pydantic/

from meutils.pipe import *
from langchain.output_parsers import PydanticOutputParser
from pydantic import BaseModel, Field
from typing import List


# 使用Pydantic创建一个数据格式，表示材料
class Flower(BaseModel):
    name: str = Field(description="name of a Material")
    colors: List[str] = Field(description="the colors of this Material")


# 定义一个用于获取某种材料的颜色列表的查询
flower_query = "Generate the charaters for a random Material."

# 定义一个格式不正确的输出
misformatted = "{'name': '汽车漆', 'colors': ['粉红色','白色','红色','紫色','黄色']}"

# 创建一个用于解析输出的Pydantic解析器，此处希望解析为Flower格式
parser = PydanticOutputParser(pydantic_object=Flower)

# 使用Pydantic解析器解析不正确的输出
# parser.parse(misformatted) # 这行代码会出错
# OutputParserException: Failed to parse Flower from completion {'name': '汽车漆', 'colors': ['粉红色','白色','红色','紫色','黄色']}. Got: Expecting property name enclosed in double quotes: line 1 column 2 (char 1)

# 从langchain库导入所需的模块
from langchain.chat_models import ChatOpenAI
from langchain.output_parsers import OutputFixingParser

# 使用OutputFixingParser创建一个新的解析器，该解析器能够纠正格式不正确的输出
new_parser = OutputFixingParser.from_llm(parser=parser, llm=ChatOpenAI())

# print(new_parser)
# 使用新的解析器解析不正确的输出
result = new_parser.parse(misformatted)  # 错误被自动修正
print(result)  # 打印解析后的输出结果
# name='汽车漆' colors=['粉红色', '白色', '红色', '紫色', '黄色']

if __name__ == '__main__':
    text = """
    请根据我的自定义歌词和音乐风格来创作歌曲：中文标题：酒入愁肠
    英文标题：Wine and Sorrows

    歌词结构：
    [Intro]
    （独奏）

    [Verse 1]
    月光洒在破旧的篱笆，
    李白提壶独自彷徨。
    酒香飘逸入云端，
    笔下是豪迈的想象。

    [Chorus]
    一壶浊酒喜相逢，
    千载孤愁只杜甫懂。
    古道西风瘦马间，
    人间事，两行清泪长。

    [Verse 2]
    江水流淌带走忧伤，
    杜甫凝眉思国家。
    文章锋利剑如霜，
    心中苦，却言者无罪当。

    [Chorus]
    一壶浊酒喜相逢，
    千载孤愁只杜甫懂。
    古道西风瘦马间，
    人间事，两行清泪长。

    [Bridge]
    一千年来谁能懂，
    酒与愁是最深的重。
    历史长河波澜壮，
    诗人眼中看尽繁华空。

    [Chorus]
    一壶浊酒喜相逢，
    千载孤愁只杜甫懂。
    古道西风瘦马间，
    人间事，两行清泪长。

    [Outro]
    （独奏）

    音乐风格：
    传统民谣
    主要乐器：古筝、琵琶、笛子
    节奏：缓慢而深情
    氛围：沉郁、古典、充满历史感
    """


    class Song(BaseModel):
        title: str = Field(description="song title")
        lyrics: str = Field(description="Enter your own lyrics", examples=['[Intro]...[Verse]...[Chorus]...'])
        music_style: str = Field(description="Style of Music, Maximum 10 words", examples=['syncopated country ...'])


    song_parser = OutputFixingParser.from_llm(parser=PydanticOutputParser(pydantic_object=Song), llm=ChatOpenAI(model="kimi"))

    result = song_parser.parse(text)  # 错误被自动修正
    print(result)  # 打印解析后的输出结果



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : task_types
# @Time         : 2024/5/31 15:47
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
from enum import Enum

from meutils.pipe import *
from meutils.str_utils.json_utils import json_path

# "NOT_START", "SUBMITTED", "QUEUED", "IN_PROGRESS", "FAILURE", "SUCCESS", "UNKNOWN"

STATUSES = {
    "not_start": "NOT_START",

    "submitted": "SUBMITTED",

    "starting": "QUEUED",
    "queued": "QUEUED",
    "STARTED": "QUEUED",
    "started": "QUEUED",
    "pending": "QUEUED",
    "PENDING": "QUEUED",
    "Queueing": "QUEUED",

    "processing": "IN_PROGRESS",
    "in_progress": "IN_PROGRESS",
    "received": "IN_PROGRESS",
    "inprogress": "IN_PROGRESS",

    "succeed": "SUCCESS",
    "success": "SUCCESS",
    "succeeded": "SUCCESS",

    "fail": "FAILURE",
    "failed": "FAILURE",
    "canceled": "FAILURE",
    "FAILURE": "FAILURE",
    "failure": "FAILURE",

    "unknown": "UNKNOWN",

}


class TaskResponse(BaseModel):
    """异步任务 通用响应体"""
    task_id: Optional[str] = None

    code: Optional[int] = 0
    message: Optional[str] = None
    status: Optional[str] = "submitted"
    data: Optional[Any] = None

    # 系统水印：可以存token
    system_fingerprint: Optional[str] = None

    model: Optional[str] = None

    # created_at: int = Field(default_factory=lambda: int(time.time()))
    created_at: Union[str, int] = Field(default_factory=lambda: datetime.datetime.today().isoformat())

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        self.status = STATUSES.get((self.status or '').lower(), "UNKNOWN")

    class Config:
        # 允许额外字段，增加灵活性
        extra = 'allow'


class TaskType(str, Enum):
    # 存储
    oss = "oss"

    # 百度助手
    pcedit = "pcedit"

    # 图 音频 视频

    kling = "kling"
    kling_vip = "kling@vip"
    # api
    kling_image = "kling-image"
    kling_video = "kling-video"

    vidu = "vidu"
    vidu_vip = "vidu@vip"

    suno = "suno"
    haimian = "haimian"
    lyrics = "lyrics"

    runwayml = "runwayml"
    fish = 'fish'
    cogvideox = "cogvideox"
    cogvideox_vip = "cogvideox@vip"

    faceswap = "faceswap"

    # 文档智能
    file_extract = "file-extract"
    moonshot_fileparser = "moonshot-fileparser"
    textin_fileparser = "textin-fileparser"
    qwen = "qwen"

    watermark_remove = "watermark-remove"

    # 语音克隆 tts  Voice clone
    tts = "tts"
    voice_clone = "voice-clone"

    # OCR
    ocr_pro = "ocr-pro"

    # todo
    assistants = "assistants"
    fine_tune = "fine-tune"


Purpose = TaskType


class Task(BaseModel):
    id: Optional[Union[str, int]] = Field(default_factory=lambda: shortuuid.random())
    status: Optional[Union[str, int]] = "success"  # pending, running, success, failed

    status_code: Optional[int] = None

    data: Optional[Any] = None
    metadata: Optional[Any] = None
    # metadata: Optional[Dict[str, str]] = None

    system_fingerprint: Optional[str] = None  # api-key token cookie 加密

    created_at: int = Field(default_factory=lambda: int(time.time()))
    description: Optional[str] = None


class FileTask(BaseModel):
    id: Union[str, int] = Field(default_factory=lambda: shortuuid.random())
    status: Optional[str] = None  # pending, running, success, failed
    status_code: Optional[int] = None

    data: Optional[Any] = None
    metadata: Optional[Any] = None

    system_fingerprint: Optional[str] = None  # api-key token cookie 加密

    created_at: int = Field(default_factory=lambda: int(time.time()))

    url: Optional[str] = None


class FluxTaskResponse(BaseModel):
    id: Union[str, int] = Field(default_factory=lambda: shortuuid.random())

    """Task not found, Pending, Request Moderated, Content Moderated, Ready, Error"""
    status: Optional[Literal["Pending", "Ready", "Error", "Content Moderated"]] = None  # Ready, Error, success, failed

    result: Optional[dict] = None

    details: Optional[dict] = None  # Error才显示, 当做 metadata
    progress: Optional[int] = None

    def __init__(self, /, **data: Any):
        super().__init__(**data)

        self.details = self.details or self.result or {}

        if self.status is None and self.result:
            if status := (
                    self.result.get("status")
                    or self.result.get("task_status")
                    or self.result.get("state")
                    or self.result.get("task_state")
                    or self.result.get("detail")  # fal
                    or json_path(self.result, expr="$..status")

            ):
                if isinstance(status, list):
                    status = status[0]

                status = status.lower()
                logger.debug(status)

                if status.startswith(("pro", "inpro", "pending", "task_status_queu", "sub", "start", "run", "inqueue")):
                    self.status = "Pending"

                if status.startswith(("succ", "ok", "compl", "task_status_succ")):
                    self.status = "Ready"

                if status.startswith(("fail", "error", "cancel", "task_status_fail")):
                    self.status = "Error"

                if any(i in status for i in ("moder",)):
                    self.status = "Content Moderated"

        # fal 会误判其他渠道 todo 增强
        if (
                self.status is None
                and "queue.fal.run" not in str(self.result)
                and "fal.media" in str(self.result)  # fal 任务一般是有结果的 https://v3.fal.media/files
        ):
            self.status = "Ready"


if __name__ == '__main__':
    # print(TaskType("kling").name)
    #
    # print(TaskType("kling") == 'kling')

    # print(Task(id=1, status='failed', system_fingerprint='xxx').model_dump(exclude={"system_fingerprint"}))

    # print("kling" == TaskType.kling)
    # print("kling" == Purpose.kling)

    # print(Purpose('kling').value)
    # print(Purpose.vidu.startswith('vidu'))

    # print('vidu' in Purpose.vidu)

    # print('kling_vip' in {TaskType.kling, TaskType.kling_vip})

    # print('kling_vip'.startswith(TaskType.kling))

    # print(Purpose.__members__)
    # print(list(Purpose))
    #
    # print(Purpose.oss in Purpose.__members__)

    # , ** {"a": 1, "system_fingerprint": 1}
    response = TaskResponse(system_fingerprint="121")

    # print(response.model_dump())
    #
    # response.__dict__.update({"a": 1, "system_fingerprint": 1})
    #
    # print(response.model_dump())

    response.user_id = 1

    data = {
        "task": {
            "eta": 0,
            "reason": "",
            "status": "TASK_STATUS_QUEUED",
            "task_id": "d7197eed-54b4-40dd-a265-a736435abbf9",
            "task_type": "MINIMAX_HAILUO_02_10S_768P",
            "progress_percent": 0
        },
        "extra": {},
        "audios": [],
        "images": [],
        "videos": []
    }

    print(FluxTaskResponse(result=data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : pixverse_type
# @Time         : 2024/7/24 13:11
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import time

from meutils.pipe import *

BASE_URL = "https://app-api.pixverse.ai/creative_platform"

EXAMPLES = [
    {
        "Prompt": "太空战机高速穿过一个巨大的科幻内部通道，冲出通道飞向太空，通道尽头可以看到外面是太空大战",
        "Model": "v1",
        "Credit": 0,
        "AspectRatio": "512x288",
        "Seed": 937033649,
        "ConfigName": "t2v_0.4.1",
        "PromptToDualClips": 1,
        "Style": "realistic",
        "AutoCharacterPrompt": 0,
        "RemoveWatermark": 0,
        "TaskCnt": 2,
        "Quality": "480p",
        "Width": 512,
        "Height": 288,
        "CreationType": 1
    },
    {
        "Img": 4857092,
        # "ImgUrl": "https://media.pixverse.ai/upload/3597df25-8161-4eea-99a4-502ac2b49227.png",
        "ImgUrl": "https://dgss0.bdstatic.com/5bVWsj_p_tVS5dKfpU_Y_D3/res/r/image/2017-09-27/297f5edb1e984613083a2d3cc0c5bb36.png",
        "Model": "v2",
        "Credit": 0,
        "Seed": 1467863527,
        "ConfigName": "it2v_2.0.1",
        "Duration": 5,
        "AutoCharacterPrompt": 0,
        "RemoveWatermark": 0,
        "PromptToDualClips": 0,
        "Quality": "480p",
        "CreationType": 2,
        "Prompt": "开花过程"
    }
]


class PixverseRequest(BaseModel):
    Prompt: Optional[str] = None
    Img: int = Field(default_factory=lambda: int(time.time()))
    ImgUrl: Optional[str] = None
    CreationType: int = 1

    Model: str = 'v2'
    AspectRatio: str = '512x288'
    Quality: str = '1080p'

    ConfigName: str = "t2v_0.4.1"

    PromptToDualClips: int = 1
    Style: str = 'realistic'
    AutoCharacterPrompt: int = 0
    RemoveWatermark: int = 1
    TaskCnt: int = 2
    Width: int = 512
    Height: int = 288

    Duration: int = 8

    Credit: int = 0
    Seed: Optional[int] = None

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        if self.ImgUrl:
            pass
            self.CreationType = 2
            self.ConfigName = "it2v_2.0.1"

    class Config:
        json_schema_extra = {
            "examples": EXAMPLES
        }
